! Module      : getexposure_mod
! Authors     : Jon Rainger (MSSL)
!
! Description : This module calculates the exact exposure duratino in seconds from the start 
!               and end of exposure alerts (A513 and A514).
!                
!
! Comments    : 
!
! History : Created  : June 1998 - JFR as getstartendexp_mod.f90
!           Modified : July 22 1999 - getexposure_mod.f90

module getexposure_mod

use dal

implicit none

private

public getexposure

contains

!----------------------------------------------------------------------------------
subroutine getexposure(nphFile, exposure)
!----------------------------------------------------------------------------------

   character(len=255) :: nphFile                                     ! OM ODF Non-Periodic Housekeeping filename.
   type(DataSetT) :: nphDataSetH                                     ! Non-Periodic Housekeeping dataset handle.
   type(TableT) :: nphTableH                                         ! Non-Periodic Housekeeping table dataset handle.
   type(ColumnT), dimension(3) :: nphColumnH                         ! Non-Periodic Housekeeping column handle.

   integer(kind=int16), dimension(:), pointer :: typeP               ! Type pointer.
   integer(kind=int32), dimension(:), pointer :: par2P               ! Par2 pointer.
   integer(kind=int32), dimension(:), pointer :: ftCoarseP           ! FTCoarse pointer
   character(len=255) :: fileName                                    ! Input file filename.

   integer(kind=int32) :: nRows                                      ! Number of rows in binary table extension.
   integer(kind=int32) :: i                                          ! Counter.
   integer(kind=int16) A513                                          ! A513 Alert code.
   data A513 /Z'A513'/
   integer(kind=int16) A514                                          ! A514 Alert code.
   data A514 /Z'A514'/
   logical(kind=bool) :: startFound                                  ! Flag.
   logical(kind=bool) :: endFound                                    ! Flag.
   integer(kind=int32) :: startTimeStamp                             ! Start time stamp.
   integer(kind=int32) :: startFtCoarse                              ! Start FTCOARSE time.
  
   integer(kind=int32) :: endTimeStamp                               ! End time stamp.
   integer(kind=int32) :: endFtCoarse                                ! End FTCOARSE time.
 


   real(kind=single) :: exposure                                     ! Actual exposure in seconds.

! Read in OM ODF Non-Periodic Housekeeping filename.

   nphDataSetH = dataSet(nphFile, read)

! Determine file type from file name in primary header.

   fileName = trim(stringAttribute(nphDataSetH, "FILENAME"))

   if (fileName(25:27) == "NPH") then

! Get a handle on the non-periodic housekeeping binary table extension.

      nphTableH = table(nphDataSetH, "OMNPH1")

! Timestamps for begining and end of exposure are found through TYPE's 'A513' and 'A514' in PAR2.


      nphColumnH(1) = column(nphTableH, "FTCOARSE", read)
      ftCoarseP => int32Data(nphColumnH(1))

      nphColumnH(2) = column(nphTableH, "TYPE", read)
      typeP => int16Data(nphColumnH(2))

      nphColumnH(3) = column(nphTableH, "PAR2", read)
      par2P => int32Data(nphColumnH(3))

      nRows = numberOfRows(nphTableH)

      i = 0
      startFound = .false.
      endFound = .false.
      do while ((i < nRows) .and. ((.not. startFound) .or. (.not. endFound)))
         if (typeP(i) == A513) then
            startFtCoarse = ftCoarseP(i) 
            startTimeStamp = par2P(i)
            startFound = .true.
         else if (typeP(i) == A514) then
            endFtCoarse = ftCoarseP(i)
            endTimeStamp = par2P(i)
            endFound = .true.
         end if
         i = i + 1
      end do

      if (endTimeStamp > startTimeStamp) then
         exposure = (endTimeStamp - startTimeStamp)/1024.0
      else
         exposure = (2.0**24 - startTimeStamp + endTimeStamp) /1024.0
      end if

!      obtStart = real(ibits(startTimeStamp, 10, 14)) + real((ibits(startTimeStamp, 0, 10) / 1024.0))
!      obtEnd = real(ibits(endTimeStamp, 10, 14)) + real((ibits(endTimeStamp, 0, 10) / 1024.0))

!      obtStart = obtStart + real(ibits(startFtCoarse, 14, 18)*(2.0**14.0))
!      obtEnd = obtEnd + real(ibits(endFtCoarse, 14, 18)*(2.0**14.0))

! Release handles on column and table.

      call release(nphColumnH(1))
      call release(nphColumnH(2))
      call release(nphColumnH(3))
      call release(nphTableH)     

   else

! Incompatible file.

      call fatal("badFileType","Value for keyword FILENAME is incompatible")
   end if

! Release handles on data.

   call release(nphDataSetH)
      
 end subroutine getexposure
  
end module getexposure_mod
