!**************************************************************
! module omphotometry_mod (10th July 2001)
!
! Module containing routines, etc,
! for doing aperture photometry on each source in the list. It has been added
! because the counts that initially computed for each source (at the same
! time that the centroided positions for each source are obtained) is not
! consistent with the CAL calibration of source counts to magnitudes.
! It also sets the source confusion flag and the source
! quality flag. 
!
! 2013-11-12 adding parameter sigmaPoisson to the subroutine 
! BackgroundCountsPerPixel
! 2013-12-04 applying the limit of 0.97 counts per frame 
! for the calculation of source photometry
! 2013-12-04 corrected the calculation of the source and background count rates
! in the case of the mosaiced image (the errors were overestimated)
!**************************************************************
module omPhotometry_mod
    
    use param
    use cal
    use dal
    use OmAllocation_mod      ! For memory allocation
    use omconstants_mod
    use OmSourceparam_mod
    use OmPhotometryDefs_mod
    use OmNumerics_mod
    use OmSkyBackground_mod   ! To compute sky background
    use OmCoinccorr_mod       ! For coincidence-loss corrections   
    use OmSmoothImage_mod
    use omellipse_mod    


interface GetExtendedSourceCounts
    module procedure GetExtendedSourceCounts1, &
                     GetExtendedSourceCounts2
end interface


interface GetExtendedCountRate
    module procedure GetExtendedCountRate1,  &
                     GetExtendedCountRate2
end interface





interface GetBackgroundMode
    module procedure GetBackgroundMode1, &
                     GetBackgroundMode2, &
                     GetBackgroundMode3, &
                     GetBackgroundMode4


end interface
interface GetCounts
    module procedure GetCounts1, GetCounts2
end interface

interface GetBackgroundCounts
    module procedure GetBackgroundCounts1


end interface


interface AperturePhotometry
module procedure AperturePhotometry1, &
                 AperturePhotometry2, &
                 AperturePhotometry3, &
                 AperturePhotometry4


end interface AperturePhotometry
   
interface ComputeSourceRateError
   module procedure ComputeSourceRateError1, &
                    ComputeSourceRateError2

end interface ComputeSourceRateError





interface setQualityFlag
    module procedure setQualityFlag1, setQualityFlag2
end interface setQualityFlag



type(SourceParameters), pointer, private :: currentSource => null()
       
real(single), private, parameter :: backgroundSigma = 2.5
real(single), private :: areaRatio = 3.0  ! WAS 3.0
integer, parameter, private :: minData = 100 ! was 299


!**********************************************************
!
!
!***********************************************************


!****************************************************************
!
! The following are only used when there is an exposure image
!
!****************************************************************

real(double), private :: meanExposure, minExposure, maxExposure
integer, private :: numExposurePixels

 
real(single), private :: exposureConstant = 1.0

contains               

!*************************************************************
! Sets the number of pixels to be used in the background 
! determination to be ratio * the number of pixels in the
! aperture  
!
!*************************************************************
subroutine setAreaRatio( ratio )
    real(single), intent(in) :: ratio

    areaRatio = ratio
end subroutine setAreaRatio

!*************************************************************
! 
!*************************************************************
real(single) function getAreaRatio()
   

    getAreaRatio = areaRatio

end Function getAreaRatio

!***************************************************************************
! If want to use an exposure image then need to set it
! using this routine 
!
!
!***************************************************************************/
subroutine setExposureImage( image, constant )
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: constant
    exposureImage => image
    exposureConstant = constant

end subroutine setExposureImage

!****************************************************************************************
! subroutine AperturePhotometry
! Performs aperture photometry on a set of sources!
! using a circular aperture of a given pixel radius
! Parameters-
!     nx and ny - the dimensions of the images
!     image - the image
!     sourceQualityImage - describes the quality of each image pixel
!     photometryRadius - The aperture radius for doing the photometry
!     noSources - number of sources
!     xSource - array of source x-positions
!     ySource - array of source y-positions
!     psfXwidth - array holding the computed x extents of the sources (pixels).
!     psfYwidth - array holding the computed y extents of the sources (pixels).y
!     fSource - array to hold the counts for each source.
!               Contains the Jon Mittaz counts on entry
!               and where possible these will be replaced 
!               by the aperture photometry values.!
!     fBackGround - array to hold the background 
!               counts for each source. 
!               Contains the Jon Mittaz counts on entry
!               and where possible these will be replaced 
!               by the aperture photometry values.
!     sourceExtendedFlag - array containg either 0 (not extended source)
!                or 1 (extended source)
!     sourceConfusionFlag - array containing either 1 - no source within
!                           2, source within aperture but outside minimum aperture
!                           radius, or 3 - source within minimum apertuire radius.
!     sourceQualityFlag - Set to 1 if source region has a bad-quality pixel- 0 otherwise
!     sourceApertureRadius - the aperture radius that was used for     
!                      each source 
!****************************************************************************************

subroutine AperturePhotometry1(   &
    nx1,                     &  ! The x-extent of the images 
    ny1,                     &  ! The y-extent of the images 
    image,                   &  ! The image (including background) 
    backgroundImageIn,       &  ! The background image (if any)
    levelImage,              &  ! The source number image
    sourceQualityImage,      &  ! The pixel-quality image array 
    photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
    noSources,               &  ! The number of sources
    source)                     ! Array of type SourceParamters structures

    ! Input and unmodified variables
    integer(int16), intent(in) :: nx1, ny1                      
    real(single), pointer              :: image(:, :)   
    real(single), pointer    :: backGroundImageIn(:, :)   
    integer(int16), pointer           :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
    !real(single), intent(in) :: photometryRadius       
    real(single), intent(inout) :: photometryRadius       
    integer (int16), intent(in)                      :: noSources

    ! The following structure is defined in  "omdetectsource_mod.f90"
    ! It stores all the parameters associated with a source 
   
    type(SourceParameters), pointer :: source(:)       

    backGroundImage => backgroundImageIn 
    !write(*,*) "L226 AperturePhotometry1: calling AperturePhotometry2"
    call AperturePhotometry2(    &
        nx1,                     &  ! The x-extent of the images 
        ny1,                     &  ! The y-extent of the images 
        image,                   &  ! The image (including background) 
        levelImage,              &  ! The source number image
        sourceQualityImage,      &  ! The pixel-quality image array 
        photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
        noSources,               &  ! The number of sources
        source)                     ! Array of type SourceParamters structures

    nullify(backgroundImage)

end subroutine AperturePhotometry1
!****************************************************
!
!
!****************************************************
subroutine AperturePhotometry2(   &
    nx1,                     &  ! The x-extent of the images 
    ny1,                     &  ! The y-extent of the images 
    image,                   &  ! The image (including background) 
    levelImage,              &  ! The source number image
    sourceQualityImage,      &  ! The pixel-quality image array 
    photometryRadius,        &  ! The aperture radius (pixels) to be used for doing photometry
    noSources,               &  ! The number of sources
    source)                     ! Array of type SourceParamters structures

    ! Input and unmodified variables
    integer(int16), intent(in) :: nx1, ny1                      
    real(single), pointer              :: image(:, :)      
    integer(int16), pointer           :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
    !real(single), intent(in) :: photometryRadius       
    real(single), intent(inout) :: photometryRadius       
    integer (int16), intent(in)                      :: noSources

    ! The following structure is defined in  "omdetectsource_mod.f90"
    ! It stores all the parameters associated with a source 
   
    type(SourceParameters), pointer :: source(:)       
 
    ! Local variables

    integer(int32)           :: i, k, status, type    
    real(single) :: maxRadius         
    real(single) :: backGroundCountsError
    real(single) :: sigma    

    ! NB Set minSignificance to zero if no counts extrapolation
    ! to be used ! NB until the CAL calibration data is in place
    ! the full aperture will be used on all sources, unless
    ! another source lies within the aperture. If the source 
    ! significance is less than this value then
    ! the photometric counts will be extrapolated using
    ! the point-spread-function
    real(single), parameter :: minSignificance = 0.0  
    integer (int32) :: index

    integer :: i0, j0, j
    !**************************************************************************
    ! 2013-11-06 protecting photometryRadius from changing on a 64-bit computer
    !**************************************************************************
    real(single) :: photometryRadiusSafe
    photometryRadiusSafe=photometryRadius

!integer, save :: count = 0       
    !write(*,*) "L293 aperturePhotometry2  noSources=", noSources
    
    if (noSources .lt. 1) return ! Return if no sources

    !write(*,*) "=============================================="
    !ttttttttttttttttttt
    !do i=1, noSources
    !   write(*,*) "L229 inside  i=", i, " x=", source(i)%xPos, " y=", source(i)%yPos
    !enddo
    !tttttttttttt

    !ttttttt
    !write(errstr,'(a)') "L292 Performing aperture photometry2 on the sources"
    !call message(AppMsg, Verbose)
    !ttttttt

    ! Initialise arrays, etc
    apertureRadius = photometryRadius
        
    call Initialise(noSources, nx1, ny1)

    
    !******************************************************************
    !
    !
    !
    !******************************************************************
     
    i0=int(source(1)%xpos)
    j0=int(source(1)%ypos)
    
    !write(*,*) "L326 i0=",i0, " j0=", j0
    

    CCDImage => image  ! Assign to input image
    qualityImage => sourceQualityImage ! The quality image (false=good pixels, true=bad pixels) 
    sourceNumImage => levelImage ! The source number image
    
    !write(*,*) "L333 CCDImage(30, 449)=", CCDImage(30, 449)    
    ! The procedure refineNumImage puts the source number into the 
    ! circle of apertureRadius
    call refineSourceNumImage(source, nosources, sourceNumimage)    
   
    ! Set the minimum aperture radius to half the required aperture
    minimumApertureRadius = photometryRadius * 0.5
    fullApertureArea = PI * photometryRadius * photometryRadius  

    
    !**********************************************************
    ! Obtain an index of the source positions in
    ! increasing x-pixel position. NB xSource is not
    ! sorted. sortIndex(i) will give the index of the 
    ! array element that gives the positions in increasing
    ! x-position. This is used for obtaining the distance
    ! from one source to its nearest neighbour. 
    !***********************************************************
    call IndexSort(source%xPos, sortIndex, int(noSources, int32))  

    ! Compute the mode of the image, ignoring source regions

    backgroundCountsMode = ComputeBackgroundMode(image, &
                     sourceNumImage, nx, ny, backgroundSigma, sigma)

    backgroundCountsModeError = sigma
    
    !********************************************************
    ! sourceCoords will contain the position of the sources
    ! in increasing x-position- this array is used to obtain
    ! the maximum-sized aperture for doing photometry.
    !********************************************************
    
    do i = 1, noSources
       sourceCoords(i)%x = source(sortIndex(i))%xPos - 0.5   
       sourceCoords(i)%y = source(sortIndex(i))%yPos - 0.5  
       sourceCoords(i)%extension = source(sortIndex(i))%extensionFlag
       !        sourceCoords(i)%maxCounts =  abs( source(sortIndex(i))%rawCountRate )
       ! objectWidth will store the maximum of the x and y extents of the sources
       ! * 3, or if this is less than the aperture radius, to the aperture radius.                       
       objectWidth(i) = photometryRadius   
       
    end do
    
    ! Loop through all the sources doing aperture photometry on each one (if possible)  
    k = 0

!write(*, *) "L379 ********************************************************************************"
!write(*, *) "L389 ********************************************************************************"


    do i = 1,  noSources ! Remember photometry on sources is conducted in increasing x-order        
       index = sortIndex(i)
       sourceNumber = source(index)%number
       !**************************************************************************
       ! 2013-11-06 protecting photometryRadius from changing on a 64-bit computer
       !**************************************************************************        
       photometryRadius=photometryRadiusSafe
       !write(*,*) "---"
       
       currentSource => source(index)         
       currentSource%number = sourceNumber
       !write(*,*) "L394 i=", i, " x=", currentSource%xPos, " y=", currentSource%yPos      
       if (currentSource%extensionFlag .eq. 0 ) then  
          
          ! Point-source photometry using circular aperture  
          !write(*,*) "L398 photometry radius=", photometryRadius        
          apertureRadius = photometryRadius    
          !
          !********************************************
          ! Find the aperture radius that can be used
          ! for the photometry. If type is 1 then
          ! can use the required aperture, 2 - have
          ! to use a smaller one, 3 can't do photometry
          !********************************************
          
          if (currentSource%apertureRadius > coincLossradius) then
             type = 5
             !write(*,*) "L408 setting type to 5" 
          else if (currentSource%apertureRadius .gt. 0.0) then
             !*********************************************************************
             ! 2013-10-03 vny: Why the function GetMaximumAperture is called here?
             !*********************************************************************
             maxRadius = GetMaximumAperture(noSources, i, type)                          
             maxRadius = currentSource%apertureRadius
             type = 1
             !write(*,*) "L416 setting the type to 1 (can use the requared aperture) maxRadius=", &
             !     maxRadius            
          else
             !***********************************************************
             ! Basically the default in imaging chain
             !***********************************************************
             maxRadius = GetMaximumAperture(noSources, i, type)             
             if (btest(currentSource%qualityFlag, 4)) then                
                maxRadius = coincLossRadius                 
             end if
          end if
          !write(*, *) "L440 type=", type, " apertureradius=", maxradius  
                                
          
          select case (type)    
          case (1, 2, 4)     ! 1 - No other objects within aperture, 
             ! 2 - Another object within aperture, but outside 
             ! minimum aperture radius- use aperture that 
             ! excludes this source
             
             
             maxradius = max(maxRadius, coincLossRadius / 2.0)
             ! Point-source photometry
             
             !write(*,*) "L441 maxRadius=", maxRadius, " coicLossRadius=", coincLossRadius

             if(associated(exposureImage) ) then 
                !write(*,*) "L444 calling PointSourcePhotometry4"
                call PointSourcePhotometry4( currentSource, maxRadius)                                     
             else
                !write(*,*) "L447 calling PointSourcePhotometry1"
                call PointSourcePhotometry1(currentSource, maxRadius) 
             end if
             
             !**************************************************************************
             ! 2013-11-06 protecting photometryRadius from changing on a 64-bit computer
             !**************************************************************************        
             photometryRadius=photometryRadiusSafe               
             
             if (type .eq. 4) then ! Source lies near an image edge
                fullApertureArea = PI * maxRadius * maxRadius
                
                !write(*,*) "L462 source lies near the image edge rawCountRate=", currentSource%rawCountRate, &
                !     " apertureArea=", apertureArea, " fullApertureArea=", fullApertureArea
                !write(*,*) "L464 rawCountRateErr=", currentSource%rawCountRateErr
                currentSource%rawCountRate = currentSource%rawCountRate &
                     * fullApertureArea / apertureArea 
                currentSource%rawCountRateErr = currentSource%rawCountRateErr &
                     * fullApertureArea / apertureArea 
                !write(*,*) "L469 rawBackgroundCountRate=", currentSource%rawBackgroundCountRate, &
                !     " backgroundCountsError=",backGroundCountsError
                !write(*,*) "L471 source rawbackgroundCountRateErr=", currentSource%rawbackgroundCountRateErr
                !*******************************************************************************
                ! 2013-11-15 the initial value of backgroundCountserror is not defined (fixing)
                !*******************************************************************************
                currentSource%rawBackGroundCountRate = currentSource%rawBackGroundCountRate &
                     * fullApertureArea / apertureArea 
                backGroundCountsError = backGroundCountsError * fullApertureArea &
                     / apertureArea 
                !currentSource%rawbackgroundCountRateErr =  backGroundCountsError &
                !     / currentSource%exposure
                currentSource%rawbackgroundCountRateErr =  currentSource%rawbackgroundCountRateErr &
                     * fullApertureArea / apertureArea 
                !write(*,*) "L483 currentSource%rawbackgroundCountRateErr =", currentSource%rawbackgroundCountRateErr 
                currentSource%apertureRadius = maxRadius                     
             end if
             
             
             !***********************************************************************
             !
             ! If using an exposure image then multiply significance by sqrt(mean exposure)
             !
             !*************************************************************************
             !                    if (associated(exposureImage) ) then
             !                        call scaleErrors( currentSource )        
             !                    end if
             
             
             currentSource%ApertureRadius = maxRadius * real(binX)
                                
             ! Correct the counts for coincidence losses
             ! NB The corrected count rates are stored in source
             ! members correctedCountRate, correctedCountRateErr,
             ! and correctedBackgroundRateErr   
             
             ! 2013-10-14 using deadFrac
             !call PointSourceCoincLossCorr( currentSource, 12.0, frameTime )
             !write(*,*) "L504 calling PointSourceCoincLossCorr"
             call PointSourceCoincLossCorr( currentSource, 12.0, frameTime, deadFrac )    
             !write(*,*) "L506 finished PointSourceCoincLossCorr for the case 1, 2 or 4" 
          case (3)   
             
             !write(*,*) "L509 case 3: reducing maxRadius"      
             maxRadius = coinclossRadius / 2.0                    
             
             if(associated(exposureImage) ) then         
                !write(*,*) "L513 calling PointSourcePhotometry4"
                call PointSourcePhotometry4(currentSource, maxRadius) 
             else
                !write(*,*) "L516 calling PointSourcePhotometry1"
                call PointSourcePhotometry1(currentSource, maxRadius) 
             end if             
             currentSource%ApertureRadius = maxRadius * real(binX)
                         
             !***********************************************************************
             !
             ! If using an exposure image then multiply significance by sqrt(mean exposure)
             !
             !*************************************************************************
             !                    if (associated(exposureImage) ) then
             !                        call scaleErrors( currentSource )
             !                    end if
             
             ! 2013-10-14 using deadFrac
             !write(*,*) "L531 calling PointSourceCoincLossCorr"
             !call PointSourceCoincLossCorr(currentSource, 12.0, frameTime)     
             call PointSourceCoincLossCorr(currentSource, 12.0, frameTime, deadFrac)   
             !type = 3
             
          case (5)  ! Extended photometry on point-source (ie aperture bigger
             ! than coincidence-loss aperture
             !write(*,*) "L538 type=5:  calling PointSourceCoincLossCorr"
             call PointSourcePhotometry2(currentSource, currentSource%apertureRadius)
             
     
          end select
          
       else

          ! Extended source photometry
          ! The counts (source + background) are corrected
          ! for coincidence-losses  
                    
          if(associated(exposureImage) ) then         
             
             call ExtendedSourcePhotometry4(currentSource) 
          else             
             if (currentsource%apertureRadius .le. 0.0) then
                !write(*,*) "L556 cGetExtendedSourceCounts"
                call GetExtendedSourceCounts(currentSource, coincLossRadius)
                
             else
                ! For elliptical apertures
                !write(*,*) "L561 cGetExtendedSourceCounts"
                call GetExtendedSourceCounts(currentSource, coincLossRadius,   &
                     currentSource%apertureRadius * currentSource%axialRatio, &   ! semi a
                     currentSource%apertureRadius, &                              ! semi b
                     currentSource%alpha)
             end if
          end if
          
          type = 0
       end if
       
       !if (count .lt. 20) then
       !count = count + 1
       
       !source(index)%semia = source(index)%apertureRadius / 2.0
       !source(index)%semib = source(index)%semia
       
       ! Set the source confusion flag - 0 isolated source
       ! 1 Source within aperture radius and greater than minimum aperture radius
       ! 2 Source within minimum aperture radius
           
       nullify(currentSource)        
       
       !100 format(i8, 8e12.4, f12.2, i8, i8, f8.1, i6)
    end do
        
    !call SaveRealImage1(ccdimage, nx1, ny1, "ccdimage.fits", "test")
        
    nullify(ccdImage, backgroundImage, qualityImage, sourceNumImage)     
    
    
    deallocate(sortIndex, stat = status)
    nullify(sortIndex)
    call CheckDeallocation(status, "sortIndex (Subroutine AperturePhotometry)")

    deallocate(sourceCoords, stat = status)
    nullify(sourceCoords)
    call CheckDeallocation(status, "sourceCoords (Subroutine AperturePhotometry)")

    deAllocate(backgroundPixel, stat = status)
    nullify(backgroundPixel)
  
    call CheckDeallocation(status, "backgroundPixel (Subroutine AperturePhotometry)")


    deAllocate(objectWidth, stat = status)
    call CheckDeallocation(status, "objectWidth (Subroutine AperturePhotometry)")
    nullify(objectWidth)

    
end subroutine AperturePhotometry2




!*********************************************
!
! Function GetApertureCounts
! Computes the counts within a given aperture
! Returns the value via the function name
!
!*********************************************
real(double) function GetApertureCounts(&
                    x0,  &  ! The x-pixel coordinate of the centre of the aperture
                    y0,  &  ! The y-pixel coordinate of the centre of the aperture
                radius)  ! The aperture radius (pixels) for summing the counts
               
    implicit none

    !*********************************************
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: radius
    
    ! Local variables
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
   

    apertureRadius = radius        
           
    ! Determine upper and lower array indices for
    ! summing counts
   
    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)
 
    GetApertureCounts = 0.0_double 

    apertureArea = 0.0_double

    ! Sum the pixels in the strip round the centre





    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   

    do i = i1, i2
        do j = j1, j2         
            GetApertureCounts = GetApertureCounts + PixelValue1(x0, y0, i, j)
            

        
        end do
    end do

  
end function GetApertureCounts       
!*********************************************
! Function GetApertureCounts1
! Computes the counts within a given aperture
! Returns the value via the function name
! This version for extended sources only
!*********************************************
real(double) function GetApertureCounts1(&
                    x0,  &  ! The x-pixel coordinate of the centre of the aperture
                    y0,  &  ! The y-pixel coordinate of the centre of the aperture
                radius  )  ! The aperture radius (pixels) for summing the counts
             
    implicit none

    !*********************************************
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: radius
   
    !********************************************
    ! Local variables
    !*******************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
   

    apertureRadius = radius        
        
    ! Determine upper and lower array indices for
    ! summing counts

    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)
 
    GetApertureCounts1 = 0.0_double 

    apertureArea = 0.0_double




    !***************************************************
    ! Sum the pixels in the strip round the centre
    !***************************************************
     
  
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
   

    do i = i1, i2
        do j = j1, j2         
            GetApertureCounts1 = GetApertureCounts1 + PixelValue2(i, j)
            
       
  
        end do
    end do

  
end function GetApertureCounts1       
!********************************************
! Function PixelValue1
! Purpose - returns the value of the pixel
! at the given position weighted by the area
! of the aperture contained within the pixel
!*********************************************
real(double) function PixelValue1(x0, y0, iIndex, jIndex, areaOut)
    implicit none   
    real (single), intent(in) :: x0, y0
    integer (int32), intent(in) :: iIndex, jIndex
    real(double), intent(out), optional :: areaOut  
 
    real (single) :: r1, r2, r3, r4
    real (single) :: x, y, x1, x2, y1, y2
    real (single) :: minRadius, maxRadius
    real(double) :: area    
    real(single) :: fCounts
   
     
    x = real(iIndex)  
    y = real(jIndex) 
 
    x1 = x - x0
    y1 = y - y0
    x2 = x1 + 1.0
    y2 = y1 + 1.0

    r1 = sqrt(x1 * x1 + y1 * y1)
    minRadius = r1
    maxRadius = r1
    !write(*,*) "L789 x1=", x1, " y1=", y1, " x2=", x2, " y2=", y2
    r2 = sqrt(x2 * x2 + y1 * y1)
    minRadius = min(minRadius, r2)
    maxRadius = max(maxRadius, r2)
    !write(*,*) "L793 minRadius=", minRadius, " maxRadius=", maxRadius
    r3 = sqrt(x1 * x1 + y2 * y2)
    minRadius = min(minRadius, r3)
    maxRadius = max(maxRadius, r3)
    !write(*,*) "L797 minRadius=", minRadius, " maxRadius=", maxRadius, " r3=", r3

    r4 = sqrt(x2 * x2 + y2 * y2)
    minRadius = min(minRadius, r4)
    maxRadius = max(maxRadius, r4)
    !write(*,*) "L802 minRadius=", minRadius, " maxRadius=", maxRadius, " r4=", r4
    !write(*,*) "L803  apertureRadius=", apertureRadius

    if (apertureRadius .le. minRadius) then
        PixelValue1 =  0.0_double
        !write(*,*) "L807 PixelValue1=", PixelValue1
        if(present(areaOut) ) then
            areaOut = 0.0
        end if

        return 
    end if
    
    if( isNullValue(CCDImage(iIndex, jIndex)) ) then
       
        fCounts = 0.0
        !write(*,*) "L818 fCounts=", fCounts
    else
        fCounts = CCDImage(iIndex, jIndex)
        !write(*,*) "L821 fCounts=", fCounts, " iIndex=", iIndex, " jIndex=", jIndex
        !*********************************************************************
        ! Maybe using an exposure map- get the mean, min and max  exposure
        !********************************************************************  
        call getExposure(iIndex, jIndex)
          
        ! Check that the pixel is okay to use
        if (associated(qualityImage)) then    
            if (qualityImage(iIndex, jIndex)) then ! Bad pixel- set source quality flag
                call setQualityFlag    
            end if
        end if


    end if  

    
    if (apertureRadius  .ge. maxRadius) then
        apertureArea = apertureArea + 1.0_double      
        PixelValue1 = fCounts
        area = 1.0
        !write(*,*) "L840 pixelValue1=", PixelValue1, " area=", area
    else
        area = EffectivePixelArea(x1, y1)  ! Pixel area (1 if full pixel)
        pixelValue1 = area * fCounts
        apertureArea = apertureArea + area   
        !write(*,*) "L845 pixelValue1=", PixelValue1, " area=", area, " apertureArea=", &
        !     apertureArea
    end if

    if(present(areaOut) ) then
        areaOut = area
    end if


 
end function PixelValue1
!********************************************
! Function PixelValue2
! Purpose - For extended sources
!*********************************************
real(double) function PixelValue2(iIndex, jIndex)
       
    
    integer(int32), intent(in) :: iIndex, jIndex
  
  
    ! Check that the pixel is okay to use
    if (associated(qualityImage)) then 
        if (qualityImage(iIndex, jIndex)) then ! Bad pixel- set source quality flag
            call setQualityFlag

        end if
    end if
    
    apertureArea = apertureArea + 1.0_double      

    if( isNullValue(CCDImage(iIndex, jIndex) ) ) then

        PixelValue2 = 0.0_double
    else
    
        PixelValue2 = CCDImage(iIndex, jIndex)
 

        !*********************************************************************
        ! Maybe using an exposure map- get the mean, min and max  exposure
        !********************************************************************  
        call getExposure(iIndex, jIndex)
  
   end if
 
end function PixelValue2
!**************************************************
! Function EffectivePixelArea
! Computes the area of a pixel contained
! within the aperture and returns the value
! via the name
! This will shortly be replaced by a better routine
!***************************************************
real(double) function EffectivePixelArea(x1, y1)
  
    real(single), intent(in) :: x1, y1
    real(single) :: radiusSqr
    real(single) :: xPos, yPos
    
    integer(int32) :: nIn
    integer(int32) :: i, j
     

    real(single), save :: deltaY(1 : 99)
    
    forall (j = 1 : 99) deltaY(j) = y1 + 0.01 * real(j)

    nIn = 0

    do i = 2, 100

        xPos = x1 + 0.01 * real(i - 1)
        do j = 1, 99     
            ypos = deltaY(j)        
            radiusSqr = xPos * xPos + yPos * yPos 
            if (sqrt(radiusSqr) .le. apertureRadius) then                         
                nIn = nIn + 1
            end if
        end do
    end do

    EffectivePixelArea = dble(nin) / 9801_double 

end function EffectivePixelArea


!***********************************************
! ComputeSourceRateError
! Computes the source counts error and returns 
! the value via the name
! NB sourceCounts is the background-subtracted counts
! and backgroundcounts the background counts
!***********************************************
real(double) function ComputeSourceRateError1(sourceCounts, backGroundCounts, exposure)
    real(double), intent(in) :: sourceCounts       ! The background-subtracted source counts
    real(double), intent(in) :: backGroundCounts   ! The background counts
    real(single), intent(in) :: exposure
  
   
    !**********************************************************************************
    ! vny: 2013-10-21 The input sourceCounts are without background, but the source count error
    ! should be calculated by using the total counts in the source aperture, so we add the 
    ! background (to get total counts and add the background yet again to take into account
    ! the subtraction of the background when the source counts were calculated
    !***********************************************************************************
    ComputeSourceRateError1 = sqrt(max(0.0_double, sourceCounts + 2.0_double * &
         backGroundCounts)) / dble(exposure)    
    
end function ComputeSourceRateError1

!***********************************************
! ComputeSourceRateError2
! (vny 2013-10-15)
! Computes the source counts error 
! for the background-subtracted counts by using the background error
! estimate at the input of this routine 
!***********************************************
real(double) function ComputeSourceRateError2(sourceCounts, backGroundCounts, backGroundCountsErr, exposure)
    real(double), intent(in) :: sourceCounts       ! The background-subtracted source counts
    real(double), intent(in) :: backGroundCounts   ! The background counts
    real(single), intent(in) :: backGroundCountsErr   ! The background counts error
    real(single), intent(in) :: exposure
     
    ComputeSourceRateError2 = sqrt(max(0.0_double, sourceCounts + backgroundCounts + &
         dble(backGroundCountsErr*backGroundCountsErr))) / dble(exposure)
    
end function ComputeSourceRateError2

!*****************************************
! test error calculation using the algebraically slightly differen formula
! but it is equivalent to the use of the background error
! 2013-11-27 (Mat's formula)
!***************************************
real(double) function ComputeSourceRateError3(sourceCounts, backGroundCountRate, exposure, &
     apertureArea, bkgAreaPix)
    real(double), intent(in) :: sourceCounts       ! The background-subtracted source counts
    real(single), intent(in) :: backGroundCountRate   ! The background count rate    
    real(single), intent(in) :: exposure ! exposure time 
    real(double), intent(in) :: apertureArea ! source aperture area in pixels
    real(single), intent(in) :: bkgAreaPix ! background aperture area in pixels

    real :: bkgCounts

    bkgCounts=backGroundCountRate * exposure

    ComputeSourceRateError3 = sqrt(max(0.0_double, sourceCounts + bkgCounts * &
         dble(1.0+apertureArea/bkgAreaPix))) / dble(exposure)
    
end function ComputeSourceRateError3

!******************************************************************
! Function GetMaximumAperture
! Finds the maximum aperture size that can be used for photomery
! Returns the value via the function name
! Also return "type" -
! type - 1 = no object within aperture radius pixels
!        2 = object within aperture radius, but outside minimum aperture radius
!        3 = object within minimum aperture radius
!*******************************************************************
real(single) function GetMaximumAperture(noSources, index, type)
    integer(int16), intent(in) :: noSources        ! The number of sources    
    integer(int32), intent(in) :: index            ! Index of source 
    integer(int32), intent(out) :: type            ! Determines what kind of photometry can be performed
    TYPE(OMPoint) :: centre                   ! Coords of source
    real(single) :: minimumSeparation  ! Minimum distance between source of interest and other sources
    real(single) :: borderDistance     ! Distance to the edge of the image from the source 
    real(single) :: okayDistance       ! Minimum allowed separation between source and nearest neighbour   
    real(single) :: distance           ! Distance bewteen source and neighbour
    real (single):: d, d1
!    real(single) :: maximumCounts
    integer(int32) :: k    
   
    centre = sourceCoords(index)


    okayDistance = 2.0 * apertureRadius  ! Okay for aperture photometry if nearest star is 2 aperture radii distant
    borderDistance = 1e8
    minimumSeparation = 1e6
    GetMaximumAperture = 12.0
!    maximumCounts = 0.0
    type = 0
    
    !******************************************
    ! Checks to see if object is near the
    ! edge of the image
    !******************************************
    if (centre%x - xLeft .lt. apertureRadius) then
        borderDistance = centre%x - xLeft
        type = 4
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
        

    else if  (xRight - centre%x .lt. apertureRadius) then
        type = 4
        borderDistance = xRight - centre%x
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
        
    end if

    if (centre%y - yBottom .lt. apertureRadius) then
        type = 4
        borderDistance = min(centre%y - yBottom, borderDistance)
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
 
      
    else  if (yTop - centre%y .lt. apertureRadius) then
        type = 4
        borderDistance = min(yTop - centre%y, borderDistance)
        currentSource%qualityFlag = ibset(currentSource%qualityFlag, 7)
       
 
    end if
   

    !***************************************************
    ! Find minimum distance between current source 
    ! and sources to its left. NB sourceCoords contains
    ! the x-coordinates of the sources in increasing 
    ! order.
    !***************************************************
    if (index .gt. 1) then
       
       k = index
       
       do while (k .gt. 1)
          k = k - 1
          if (sourceCoords(k)%extension .eq. 1) cycle
          
          d = centre%x - sourceCoords(k)%x
          d = max(0.0, d)
          if (d .gt. apertureRadius + objectWidth(k) ) exit
          if (sourceCoords(k)%y .ge. centre%y) then
             d1 = sourceCoords(k)%y - centre%y
             d1 = max(0.0, d1)
             
          else
             d1 = centre%y - sourceCoords(k)%y
             d1 = max(0.0, d1)
          end if
          
          distance = sqrt(d * d + d1 * d1)
          
          if (distance + apertureRadius - objectWidth(k) .lt. minimumSeparation) then 
             minimumSeparation = distance + apertureRadius - objectWidth(k)
             !                maximumCounts = max(maximumCounts, sourceCoords(k)%maxCounts)
             
          end if
          
       end do
    end if
   
    !***************************************************
    ! Find minimum distance between current source
    ! and sources to its right.
    !**************************************************
    if (index .lt. noSources) then
      
       k = index
       do while (k .lt. noSources)
          k = k + 1
          if (sourceCoords(k)%extension .eq. 1) cycle
          
          d = sourceCoords(k)%x - centre%x
          d = max(0.0, d)
          
          if (d .gt. apertureRadius + objectWidth(k)) exit
          
          if (sourceCoords(k)%y .ge. centre%y) then
             d1 = sourceCoords(k)%y - centre%y
             d1 = max(0.0, d1)
          else
             d1 = centre%y - sourceCoords(k)%y
             d1 = max(0.0, d1)
          end if
          
          distance = sqrt(d * d + d1 * d1)
          if (distance + apertureRadius - objectWidth(k) .lt. minimumSeparation) then
             minimumSeparation = distance + apertureRadius - objectWidth(k)
             !                maximumCounts = max(maximumCounts, sourceCoords(k)%maxCounts)
             
          end if
          
       end do
    end if
    

    if (type .eq. 4) then
      
        if (minimumSeparation .ge. borderDistance) then
            GetMaximumAperture = max( real(4 / binx), borderDistance)
            return
         end if
    end if    
    
   
    !****************************************************************
    !
    ! Having reached here, minimumSeparation is the minimum
    ! distance between the current source and its nearest neighbour
    ! apertureRadius is the default aperture radius (12/binx) pixels 
    ! minimumApertureRadius is the minimum-allowed aperture radius
    !
    !
    !****************************************************************

    if (minimumSeparation .ge. okayDistance) then   
        type = 1
        currentSource%confusionFlag = 0
        GetMaximumAperture = apertureRadius
        
    else if (minimumSeparation .ge. 2.0 * minimumApertureRadius) then  
        type = 2 


        GetMaximumAperture = 0.5 * minimumSeparation   ! - apertureRadius


    

        currentSource%confusionFlag = ibset(currentSource%confusionFlag, 0 )
 
    else
        type = 3    ! Can't do photometry on
        GetMaximumAperture = 0.0
        currentSource%confusionFlag = ibset(currentSource%confusionFlag, 1 )
        
 
    end if
    

end function GetMaximumAperture


!*********************************************
!
! subroutineGetExtendedSourceCounts
! Computes the counts for an extended source
! Returns the value via the function name
! Uses the pixels assigned to the source
! on the SOURCENUM image.
!
!*********************************************
subroutine GetExtendedSourceCounts1(source, radius)

    ! Input (unmodfied) variables
    real(single), intent(in) :: radius 
    type(SourceParameters), intent(inout) :: source

    ! Local variables
    integer(int32) :: i, j, count
  
    real(single) :: x0, y0
      
    real(double) :: rawCounts, correctedCounts, countRate
    real(double) :: f
    real(double) :: backgroundRate, totalRate
    real(double) :: backgroundCounts, correctedBackgroundCounts 
    real(double) :: backgroundCountsPerPixel, backgroundRatePerPixel
    real(double) :: correction
    logical :: useBackgroundMap = .false.
    real(double) :: backgroundMode, meanExposure = 0.0_double
    integer ::  numExposurePixels = 0
!    integer, save :: count1 = 0    



 
           


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
    

    !*****************************************************************
    !
    ! Determine the corrected background count-rate per pixel
    !
    !*****************************************************************
    
 
    if (source%rawBackgroundCountRate > 0.0) then       ! Use the user-specified value

        backgroundMode = source%rawBackgroundCountRate  ! Use user background value
        backgroundRate = backgroundMode * PI * 144.0 / (dble(binX * binX) * source%exposure)

        ! The coincidence-loss correction for each background pixel
        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
        backgroundCountsPerPixel = backgroundCountsMode !* correction
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
              

    else if (associated(backGroundImage)) then          ! Using a background image
       
       useBackgroundMap = .true.
       backgroundCounts = 0.0_double
       correctedBackgroundCounts = 0.0_double
       
    else 
       
       backgroundMode = backgroundCountsMode           ! Use global image mode                 
       backgroundRate = backgroundMode * PI * 144.0 / (dble(binX * binX) * source%exposure)
      
   
        ! The coincidence-loss correction for each background pixel
        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
        
        backgroundCountsPerPixel = backgroundCountsMode * correction
        
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
        
    end if

  

    !***********************************************************************
    !
    !
    !
    !***********************************************************************

    count = 0 ! The number of pixels
    rawCounts = 0.0_double 
    correctedCounts = 0.0_double

    numExposurePixels = 0
    meanExposure = 0.0_double
    if( associated( exposureImage ) ) then
      
       source%exposure = exposureConstant
    end if


    do i = source%iMin, source%iMax
       x0 = real(i) + 0.5
     
       do j = source%jMin, source%jMax
          
          if (sourceNumImage(i, j) .eq. source%number) then ! The pixel is part of the sources

             if (associated(exposureImage )) then
                numExposurePixels = numExposurePixels + 1
                
                meanExposure = meanExposure + dble(exposureImage(i, j))
             end if
             y0 = real(j) + 0.5

             !****************************************************************
             !
             ! Compute the coincidence-loss correction for this pixel
             !
             !****************************************************************
             countRate= GetApertureCounts1(x0, y0, radius) / source%exposure                   
 
             f =  PixelValue2(i, j)
             
              
              
             rawCounts = rawCounts + f
             correctedCounts = correctedCounts + f * &
                  CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
             
             count = count + 1   
             
             if (useBackgroundMap) then       ! Use the background map
                
                backgroundCounts = backgroundCounts + backgroundImage(i, j)
                
                
             end if
          else
             !***************************************************************
             ! Added 2010-10-17 (vny) as the above piece of the code fails to
             ! to produce extended source counts
             !***************************************************************
             if (associated(exposureImage )) then
               
                numExposurePixels = numExposurePixels + 1

                meanExposure = meanExposure + dble(exposureImage(i, j))
             end if
             y0 = real(j) + 0.5

             !****************************************************************
             !
             ! Compute the coincidence-loss correction for this pixel
             !
             !****************************************************************
             countRate= GetApertureCounts1(x0, y0, radius) / source%exposure                   

             f =  PixelValue2(i, j)
             
              
              
             rawCounts = rawCounts + f
             correctedCounts = correctedCounts + f * &
                  CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
               
             count = count + 1   

             if (useBackgroundMap) then       ! Use the background map
                
                backgroundCounts = backgroundCounts + backgroundImage(i, j)
             end if
          end if

        end do
    end do


      

    if( associated(exposureImage) ) then

        meanExposure = meanExposure / dble(numExposurePixels)
        
     
        correction = meanExposure / exposureConstant
            
        backGroundCounts = backgroundCounts * correction
        rawCounts = rawCounts * correction
        correctedCounts = correctedCounts * correction 
        backgroundCountsMode = backgroundCountsMode * correction
        backgroundCountsPerPixel = backgroundCountsPerPixel * correction
        source%exposure = meanExposure
    end if

    

    if (useBackgroundMap) then            

        f = backgroundCounts / dble(count) 
        backgroundRate = f * PI * radius * radius / source%exposure
        

       

        correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
        

        backgroundCountsPerPixel = f * correction
        backgroundRatePerPixel = backgroundCountsPerPixel / source%exposure
        rawCounts = rawCounts - backgroundCounts


        

        correctedCounts = correctedCounts - backgroundCounts * correction

    else
        rawCounts = rawCounts - dble(count) *  backgroundCountsMode
        correctedCounts = correctedCounts - dble(count) * backgroundCountsPerPixel
        backgroundCounts = backgroundCountsMode * dble(count)
     
 

    end if


    ! The raw count rates
    source%rawCountRate = rawCounts / source%exposure
    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)
    source%rawBackGroundCountRate = backgroundCounts / source%exposure 
    source%corrBackGroundCountRate = backgroundRatePerPixel * dble(count)

    !************************************************
    ! 2013-10-22 compute background count rate error
    !************************************************
    source%rawBackGroundCountRateErr = sqrt(backgroundCounts) / source%exposure 
    source%corrBackGroundCountRateErr= sqrt(backgroundRatePerPixel * dble(count) * &
         source%exposure)/ source%exposure 
    
    source%corrCountRate = correctedCounts / source%exposure
    source%corrCountRateErr =  ComputeSourceRateError(correctedCounts, backGroundCounts, source%exposure)
                   
    
    !**********************************************************************
    !
    ! The source coincidence-loss correction
    !
    !**********************************************************************
    totalRate = source%rawCountRate + source%rawBackgroundCountRate
    if (totalrate .gt. 0.0) then
        source%rightAscension =  (source%corrCountRate + source%corrBackgroundCountRate) / &
           totalrate
    else
        source%rightAscension = 1.0
    end if
    ! The background coincidence-loss correction
    if (source%rawBackgroundCountRate .gt. 0.0) then
        source%declination =  source%corrBackGroundCountRate / source%rawBackGroundCountRate
    else  
        source%declination = 1.0

    end if


    !*****************************************
    !
    ! Recalculate the source significance
    !
    !*****************************************   
    source%Significance = ComputeSourceSignificance(real(rawCounts), real(backgroundCounts))   
    source%apertureRadius = nullValue
    source%psf1Correction = 0.0
    source%coincidenceLossCorr = max(0.0, (source%corrCountRate - source%rawCountRate) * &
         100.0 / source%rawCountRate) 
    if(associated( exposureImage ) ) then 
        source%exposure = meanExposure
    end if  

    


end subroutine GetExtendedSourceCounts1       
!*********************************************
! subroutine GetExtendedSourceCounts2
! For extended sources with elliptical apertures
! Computes the count-rates/etc for an extended source
! 
!*********************************************
subroutine GetExtendedSourceCounts2(source, coincLossRadius, semiA, semiB, angle)

    ! Input (unmodfied) variables
    real(single), intent(in) :: coincLossRadius, semiA, semiB, angle
    type(SourceParameters), intent(inout) :: source

    ! Local variables
    integer(int32) :: i, j
    real(single) :: x0, y0, theta
      
    real(double) :: f, rawCounts, correctedCounts, countRate
    real(double) :: backgroundRate
    real(double) :: backgroundCounts = 0.0_double, correctedBackgroundCounts
    real(double) :: correction, pixArea, totalArea
    real(single) :: error    
    logical :: useBackgroundMap = .false.
    logical :: useImage = .false.

    real(single) :: bkgAreaPix

!integer, save :: count = 0
 
    ! Determine the corrected background count-rate per pixel using 
    if (angle .lt. 0.0) then
        theta = angle + 180.0
    else
        theta = angle
    end if




    if (source%rawBackgroundCountRate > 0.0) then ! Using a fixed background mode to get background counts
        backgroundCounts = source%rawBackgroundCountRate * PI * semiA * semiB
      
    else if (associated(backGroundImage)) then ! Using a background image
        useBackgroundMap = .true.

    else ! Using the mode from the image

        
        backgroundCounts = GetBackGroundMode2( &
                     source%xPos,              &   ! The centre x-pixel coordinate of the aperture
                     source%yPos,              &   ! The centre y-pixel corrdinate of the aperture 
                     source%innerBckRadius,    &   ! The aperture inner radius
                     source%outerBckRadius,    &   ! The aperture outer radius
                     source%axialRatio,        &   ! The ellipse axial ratio
                     theta,                    &   ! The ellipse position angle
                     !error)      ! The error
                     error,                    &      ! The error
                     bkgAreaPix)
        useImage = .true.

    end if    


       


 
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
    
    rawCounts = 0.0_double
    correctedCounts = 0.0_double
    totalArea = 0.0_double
  
  
    do i = max(0, int(floor(source%xPos) - ceiling(semiA)) - 1 ),  &
                min(nx - 1, int(ceiling(source%xPos) + semiA + 0.1) + 1) 
        x0 = real(i) + 0.5
     
        do j = max(0, int(floor(source%yPos) - ceiling(semiA) ) - 1),  &
                     min(ny - 1, int(ceiling(source%yPos) + semiA + 0.1) + 1) 
       
                            
            y0 = real(j) + 0.5 ! 
            pixArea = PixelArea(x0 + 0.5, y0 + 0.5, source%xPos, source%yPos, semiA, semiB, theta)

            if (pixArea <= 0.0) cycle
            totalArea = totalArea + pixArea
            ! Compute the coincidence-loss correction for this pixel
            countRate= GetApertureCounts1(x0, y0, coincLossRadius) / source%exposure 
            if (pixArea .lt. 1.0) then                  
                f =  PixelValue2(i, j) * pixArea            

                if (useBackgroundMap) &                  
                    backgroundCounts = backgroundCounts + backgroundImage(i, j) * pixArea
         
            else
                f =  PixelValue2(i, j)            

                if (useBackgroundMap) &                  
                    backgroundCounts = backgroundCounts + backgroundImage(i, j)
         
            end if

            rawCounts = rawCounts + f
            correctedCounts = correctedCounts + f * &
                        CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
                                 
        end do
    end do


    if (useImage) backgroundCounts = backgroundCounts * totalArea



    backgroundRate = backgroundCounts / source%exposure ! Background count rate
    source%rawBackgroundCountRate = backgroundCounts / source%exposure
      
    !**********************************************************
    ! The coincidence-loss correction for the background
    !*********************************************************
    correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)
    source%corrBackGroundCountRate = backgroundRate * correction ! Corrected background rate





    correctedBackgroundCounts = backgroundCounts * correction

    ! Background-subtracted counts
    rawCounts = rawCounts - backgroundCounts
    source%rawCountRate = rawCounts / source%exposure

    !*****************************************
    ! Recalculate the source significance
    !*****************************************   
    source%Significance = ComputeSourceSignificance(real(rawCounts), real(backgroundCounts))
    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)
                        


    ! Corrected background-subtracted counts
    source%corrCountRate = (correctedCounts - correctedBackgroundCounts) / source%exposure    
    source%corrCountRateErr =  ComputeSourceRateError(correctedCounts, &
         correctedBackGroundCounts, source%exposure) 
      

             
!    source%apertureRadius = nullValue  
    source%psf1Correction = 0.0

    ! The source coincidence-loss correction
    if (source%rawCountRate .gt. 0.0) then
        source%rightAscension =  source%corrCountRate / source%rawCountRate
    end if
    ! The background coincidence-loss correction
    if (source%rawBackgroundCountRate .gt. 0.0) then
        source%declination =  source%corrBackGroundCountRate / source%rawBackGroundCountRate
    end if

    if (source%rawCountRate .gt. 0.0) then
        source%coincidenceLossCorr = source%corrCountRate / source%rawCountRate 
    else
       source%coincidenceLossCorr = 1.0
    end if
    source%rawBackGroundCountRateErr = sqrt(source%rawBackgroundCountRate * source%exposure) / source%exposure 
    source%corrBackGroundCountRateErr= sqrt(source%corrBackGroundCountRate * source%exposure)/ source%exposure 



end subroutine GetExtendedSourceCounts2      
!*********************************************
! Function GetBackGroundMode1
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation)
! 
!*********************************************
real (double) function GetBackGroundMode1( &
    levelImage,   &
    x0,           &   ! The centre x-pixel coordinate of the aperture
    y0,           &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius1, &   ! The aperture inner radius
    outerRadius1, &   ! The aperture outer radius
    !error)            ! The error
    error,        &    ! The error
    bkgAreaPix)        ! background area in pixels

    implicit none
    
    ! Input variables
    integer(int16), pointer :: levelImage(:, :)  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius1
    real(single), intent(in) :: outerRadius1
    real(single), intent(out) :: error
    
    ! 2013-10-28 vny: adding the output background area
    ! which is needed for calculation of the source error
    real(single), intent(out) :: bkgAreaPix
  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y, dx, dy, r, radius, innerRadius, outerRadius
   
    !*******************************************************
    ! 2013-11-12 vny: using the Poissonian background error
    !*******************************************************
    real(single) :: sigmaPoisson
    !*******************************************************

    !**************************************
    ! 2013-11-06 Rounding the source locations
    ! 2013-11-11 removing the rounding of the source coordinates
    !**************************************
    !real(single) :: x0rnd, y0rnd
    !integer(int32) :: i0rnd, j0rnd
    integer(int32) :: printFlag
    printFlag=0

    !i0rnd=int(x0*10.0+0.5)
    !j0rnd=int(y0*10.0+0.5)
    !x0rnd=real(i0rnd)/10.0
    !y0rnd=real(j0rnd)/10.0
    !write(*,*) "L1776 x0rnd=", x0rnd, " y0rnd=", y0rnd
    !i0rnd=int(innerRadius1*10.0+0.5)
    !j0rnd=int(outerRadius1*10.0+0.5)
    !write(*,*) "L1779 i0rnd=", i0rnd, " j0rnd=",j0rnd
 
    !**************************************
    ! 2013-11-11 removing the rounding of the source coordinates
    !**************************************
    innerRadius = innerRadius1    
    outerRadius = outerRadius1
    !
    !innerRadius=real(i0rnd)/10.0
    !outerRadius=real(j0rnd)/10.0
 
 
    i0 = int(x0)
    j0 = int(y0)

    !i0 = int(x0rnd)
    !j0 = int(y0rnd)

   !****************************
    nData = 0  
    bkgAreaPix=real(nData)
  
    !write(*, *) "L1723 x=", x0, " y=", y0, " inner radius=", innerRadius, " outer=", outerRadius  

    if (i0 .eq. 23 .and. j0 .eq. 201) then
       printFlag=1
    endif

    r = ceiling(outerRadius)
    !write(*,*) "L1802 r=", r, " i0=", i0, " j0=", j0

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    j1 = j0 - r - 1
    j1 = max(jl, j1)

    !write(*,*) "L1737 i1=", i1, " j1=", j1
    !write(*,*) "L1738 CCDImage(i1, j1)=", CCDImage(i1, j1)

    ! Determine upper and lower array indices for
    ! summing counts
    !r = ceiling(outerRadius)
        
    i2 = i0 + r + 1
    i2 = min(iu, i2)  

  
    j2 = j0 + r + 1
    j2 = min(ju, j2)
    !write(*,*) "L1750 i2=", i2, " j2=", j2
    !write(*,*) "L1751 CCDImage(i2, j2)=", CCDImage(i2, j2)
iLoop : do i = i1, i2

        x = real(i) + 0.5  
        !**************************************
        ! 2013-11-11 removing the rounding of the source coordinates
        !**************************************
        dx = x - x0
        !dx = x - x0rnd

       !if (printFlag .gt. 0) then
           !write(*,*) ".."
           !write(*,*) "L1832 i=", i, " x=", x, " dx=", dx
        !endif
        do j = j1, j2          
               
            if (associated(qualityImage)) then
                !if (printFlag .gt. 0) then
                !   if (qualityImage(i, j)) then 
                !      write(*,*) "L1839 i=", i, " j=", j, " cycling"
                !   endif
                !endif
                if (qualityImage(i, j)) cycle

            end if

            if ( levelImage(i, j) .ne. 0) then
               !if (printFlag .gt. 0) then
               !   write(*,*) "L1846 i=", i, " j=", j, " cycling"
               !endif
               cycle
            end if
                       
            y = real(j) + 0.5
            !**************************************
            ! 2013-11-11 removing the rounding of the source coordinates
            !**************************************
            dy = y - y0
            !dy = y - y0rnd

            radius = sqrt(dx * dx + dy * dy)
            !if (printFlag .gt. 0) then        
            !   write(*,*) "L1857          j=", j, " y=", y, " dy=", dy, " radius=", radius
            !endif
            if (radius .ge. innerRadius .and. radius .le. outerRadius) then         
               if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                  nData = nData + 1 
                  backGroundPixel(nData) = CCDImage(i, j)
                  !if (printFlag .gt. 0) then
                  !write(*,*) "L1799         i=", i, " j=", j, " nData=", nData, &
                  !     " value=", backGroundPixel(nData) 
                  !endif
                  
                  if (nData == maxNumPixels) then
                     exit iLoop
                  end if
               else

               end if
            end if
            
                             
        end do 

    end do iLoop

    !if (printFlag .gt. 0) then
       !write(*,*) "-----------------------"
    !endif
       
    bkgAreaPix=real(nData)

    ! Compute the median background pixel value
    !write(*, *) "L1823 ndata=", nData, " mindata=", minData
    if (nData .ge. minData) then  ! 
       !*********************************************************
       ! 2013-11-12 vny: adding an extra parameter (sigmaPoisson)
       !*********************************************************
       !GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
       ! 2013-10-15 vny: here error is the error of one measurement (sigma1), whereas the 
       ! accuracy of the background measurements corresponds to the number of measurements
       ! which is nData. Here the parameter backgroundSigma means the threshold for n-Sigma clipping
       ! (it is N)
       !********************************************************************************************
       ! NB: 2013-10-15 the routine BackgroundCountsPerPixel in the module omskybackground_mod.f90
       ! is corrected to get the backgroundSigma as the sigma of the average background and not as
       ! the standard deviation.
       ! That is, the sigma1 in the routine BackgroundCountsPerPixel is divided by sqrt(N)
       !********************************************************************************************
       GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
            error, sigmaPoisson)
       error=sigmaPoisson

       !write(*,*) "L1843 GetBackgroundMode1=", GetBackgroundMode1
    else if (associated(backgroundImage)) then  ! Use the background image
       

       nData = 0
       bkgAreaPix=real(nData)
 iLoop1 :    do i = i1, i2

            x = real(i)  
            dx = x - x0
            do j = j1, j2          
              
            
                y = real(j)
                dy = y - y0
                r = sqrt(dx * dx + dy * dy)
                if (r .ge. innerRadius .and. r .le. outerRadius) then         
              
                    nData = nData + 1 
                    backGroundPixel(nData) = backgroundImage(i, j)  
                    if (nData == maxNumPixels) then

                        exit iLoop1
                    end if  
                end if                                        
            end do
        end do iLoop1


        bkgAreaPix=real(nData)    
        if(ndata .ge. minData) then
           !*********************************************************
           ! 2013-11-12 vny: adding an extra parameter (sigmaPoisson)
           !*********************************************************
           !GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
           GetBackgroundMode1 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
                error, sigmaPoisson)
           error=sigmaPoisson
           !write(*,*) "L1881 GetBackgroundMode1=", GetBackgroundMode1
        else
          
            GetBackgroundMode1 = backgroundCountsMode
            error = backgroundCountsModeError
            !write(*,*) "L1886 GetBackgroundMode1=", GetBackgroundMode1
        end if 


    else
        GetBackgroundMode1 = backgroundCountsMode
        error = backgroundCountsModeError
        !write(*,*) "L1893 GetBackgroundMode1=", GetBackgroundMode1
    end if

end function GetBackgroundMode1       


!*********************************************
! Function GetBackGroundMode2
! For use with elliptical annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!*********************************************
real (double) function GetBackGroundMode2( &

    x0,          &   ! The centre x-pixel coordinate of the aperture
    y0,          &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius, &   ! The aperture inner radius
    outerRadius, &   ! The aperture outer radius
    axialRatio,  &   ! The ellipse axial ratio
    theta1,       &   ! The ellipse position angle
    !error)      ! The error
    error,       & ! The error
    bkgAreaPix)        ! background area in pixels

    implicit none
    
    ! Input variables  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius
    real(single), intent(in) :: outerRadius
    real(single), intent(in) :: axialRatio
    real(single), intent(in) :: theta1
    real(single), intent(out) :: error
  
    real(single), intent(out) :: bkgAreaPix

    real(single) r, semiA1, semiA2, semiB1, semiB2, theta 

  
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y

    ! 2013-11-12 adding a new parameter sigmaPoisson
    real(single) :: sigmaPoisson

!integer, save :: count = 0

    semiA1 = innerRadius * axialRatio
    semiA2 = outerRadius * axialRatio
    semiB1 = innerRadius
    semiB2 = outerRadius

    theta = theta1 * degreesToRadians
    ! Scan the annulus region and add pixel values to the 
    ! backGroundPixel array

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  
    
    bkgAreaPix=real(nData)

    i0 = int(x0)
    j0 = int(y0)
       
    r = ceiling(outerRadius * axialRatio)
   
    ! Determine upper and lower array indices for
    ! summing counts

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    i2 = i0 + r + 1
    i2 = min(iu, i2)  

    j1 = j0 - r - 1
    j1 = max(jl, j1)

    j2 = j0 + r + 1
    j2 = min(ju, j2)
 

iLoop :   do i = i1, i2

        x = real(i)  
   
        do j = j1, j2          
                    
            if (associated(qualityImage)) then
                if (qualityImage(i, j)) cycle
            end if

            if (associated(sourceNumImage)) then 
                if (sourceNumImage(i, j) .gt. 0) cycle
            end if 

           
            y = real(j)
              
            !********************************************************************
            ! Point has to be within the inner and outer ellipse
            !********************************************************************
            if (.not. PointInEllipse(x, y, x0, y0, semiA1, semiB1, theta) .and. &
                      PointInEllipse(x, y, x0, y0, semiA2, semiB2, theta)) then
          

                if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                
                    nData = nData + 1 
                    backGroundPixel(nData) = CCDImage(i, j)  
 !                  ccdimage(i, j) = -1.0
                    if (nData == maxNumPixels) then
                        exit iLoop
                    end if
                end if  
            end if
                    
            
        end do
    end do iLoop



    ! Compute the median background pixel value
    bkgAreaPix=real(nData)
    if (nData .gt. 100) then   
       !************************************************
       ! 2013-11-12 using an extra parameter sigmaPoisson
       !GetBackgroundMode2 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
       GetBackgroundMode2 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
            error, sigmaPoisson)
       error=sigmaPoisson
        

    else





        GetBackgroundMode2 = backgroundCountsMode
        error = backgroundCountsModeError


        
    end if
    bkgAreaPix=real(nData)

end function GetBackgroundMode2       

!*********************************************
!
! Function GetBackGroundMode3
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!
!*********************************************
real (double) function GetBackGroundMode3( &
    x0,          &   ! The centre x-pixel coordinate of the aperture
    y0,          &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius1, &   ! The aperture inner radius
    outerRadius1, &   ! The aperture outer radius

    !error)      ! The error
    error,        &      ! The error
    bkGAreaPix) ! background area in pixels 

    implicit none
    
    ! Input variables  
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius1
    real(single), intent(in) :: outerRadius1
    real(single), intent(out) :: error
  
    real(single), intent(out) :: bkgAreaPix

    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: innerRadius, outerRadius, x, y, dx, dy, r, radius
    integer :: iter

    ! 2013-11-12 vny: adding a new parameter sigmaPoisson
    real(single) :: sigmaPoisson
    
    ! Scan the annulus region and add pixel values to the 
    ! backGroundPixel array

    innerRadius = innerRadius1
    outerRadius = outerRadius1

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  
    bkgAreaPix=real(nData)
  


    r = ceiling(outerRadius)

    i1 = i0 - r - 1
    i1 = max(il, i1)
  
    j1 = j0 - r - 1
    j1 = max(jl, j1)



    do iter = 0, 2        
     
       
        outerRadius = outerRadius1 + real(iter * 2)
   

        ! Determine upper and lower array indices for
        ! summing counts
        r = ceiling(outerRadius)

        
        i2 = i0 + r + 1
        i2 = min(iu, i2)  

  
        j2 = j0 + r + 1
        j2 = min(ju, j2)
 
    
iLoop : do i = i1, i2

            x = real(i) + 0.5  
            dx = x - x0
            do j = j1, j2          
               
                if (associated(qualityImage)) then
                    if (qualityImage(i, j)) cycle
                end if

           
                y = real(j) + 0.5
                dy = y - y0
                radius = sqrt(dx * dx + dy * dy)
                if (radius .ge. innerRadius .and. radius .le. outerRadius) then         
                    if ( .not. isNullValue( CCDImage(i, j) ) ) then              
                  
                        nData = nData + 1 
                        backGroundPixel(nData) = CCDImage(i, j)  
                        if (nData == maxNumPixels) then

                            exit iLoop
                        end if  
                    end if
                end if
                              
            end do
        end do iLoop


        if (nData .gt. minData) exit 
    
        innerRadius = outerRadius
    end do


    bkgAreaPix=real(nData)
    ! Compute the median background pixel value
    if (nData .gt. minData) then  ! 

       !GetBackgroundMode3 = BackgroundMode(nData, backGroundPixel, backgroundSigma, error)
       ! 2013-11-12 adding a new parameter sigmaPoisson
       !GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
       GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
            error, sigmaPoisson)
       error=sigmaPoisson
    else if (associated(backgroundImage)) then  ! Use the background image

        nData = 0
iLoop1 :    do i = i1, i2

            x = real(i)  
            dx = x - x0
            do j = j1, j2          
              
            
                y = real(j)
                dy = y - y0
                r = sqrt(dx * dx + dy * dy)
                if (r .ge. innerRadius1 .and. r .le. outerRadius1) then         
              
                    nData = nData + 1 
                    backGroundPixel(nData) = backgroundImage(i, j)  
                    if (nData == maxNumPixels) then

                        exit iLoop1
                    end if  
                end if                                        
            end do
        end do iLoop1


!        GetBackgroundMode3 = BackgroundMode(nData, backGroundPixel, backgroundSigma, error)
        ! 2013-11-12 adding a new parameter sigmaPoisson
        !GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
        GetBackgroundMode3 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
             error, sigmaPoisson)
        error=sigmaPoisson
    end if

    bkgAreaPix=real(nData)


end function GetBackgroundMode3       




!*********************************************
! Function GetBackGroundMode3
! For use with circular annulii
! Computes the background counts within the given
! annulus, using the median counts/pixels obtained
! from an annulus region. 
! Return the value via the function name, and
! also the error (standard deviation) 
!*********************************************
real (double) function GetBackGroundMode4(         &
    image,                                         &
    x0,                                            &   ! The centre x-pixel coordinate of the aperture
    y0,                                            &   ! The centre y-pixel corrdinate of the aperture 
    innerRadius,                                   &   ! The aperture inner radius
    outerRadius,                                   &   ! The aperture outer radius
    !error)                                             ! The error
    error, &                                             ! The error
    bkgAreaPix)

    implicit none
    
    ! Input variables  
    real(single), pointer :: image(:, :) 
    real(single), intent(in) :: x0
    real(single), intent(in) :: y0
    real(single), intent(in) :: innerRadius
    real(single), intent(in) :: outerRadius
    real(single), intent(out) :: error
  
    real(single), intent(out) :: bkgAreaPix
 
    ! Local variables
    integer(int32) :: i, j
    integer(int32)  :: i0, j0, i1, i2, j1, j2
    integer(int32) :: ndata  ! The number of background pixels used
    real(single) :: x, y, dx, dy, r, radius
    !2013-11-12 vny: adding a new parameter sigmaPoisson
    real(single) :: sigmaPoisson   

    i0 = int(x0)
    j0 = int(y0)
    nData = 0  

  
    r = ceiling(innerRadius)

    i1 = i0 - r - 1
    i1 = max(lbound(image, 1), i1)
  
    j1 = j0 - r - 1
    j1 = max(lbound(image, 2), j1)

   
    ! Determine upper and lower array indices for
    ! summing counts
  
        
    i2 = i0 + r + 1
    i2 = min(i2, ubound(image, 1))  

  
    j2 = j0 + r + 1
    j2 = min(j2, ubound(image, 2))
 
    nData = 0
    bkgAreaPix=real(nData)

    do i = i1, i2

        x = real(i) + 0.5  
        dx = x - x0
        do j = j1, j2          
                           
            y = real(j) + 0.5
            dy = y - y0
            radius = sqrt(dx * dx + dy * dy)
            if (radius .ge. innerRadius .and. radius .le. outerRadius) then         

                if ( .not. isNullValue( image(i, j) ) ) then              
             
                    nData = nData + 1 
                    backGroundPixel(nData) = image(i, j)  
                    if (nData == maxNumPixels) then

                        exit 
                    end if  
                 end if   
             end if  
                              
        end do
    end do




    bkgAreaPix=real(nData)
    ! Compute the median background pixel value
    if (nData .gt. minData) then  ! 

       ! 2013-11-12 vny: adding a new parameter sigmaPoisson
       !GetBackgroundMode4 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, error)
       GetBackgroundMode4 = BackgroundCountsPerPixel(nData, backGroundPixel, backgroundSigma, &
            error, sigmaPoisson)
       error=sigmaPoisson

    else
        GetBackgroundMode4 = backgroundCountsMode

    end if

    bkgAreaPix=real(nData)

end function GetBackgroundMode4       





!*******************************************************
!
! Subroutine Initialise
!
!******************************************************
subroutine Initialise(noSources, nx1, ny1)
    integer(int16), intent(in) :: noSources, nx1, ny1

 
    real(single) :: calRadius
    integer(int32) :: status
    
    nx = nx1
    ny = ny1
    
    coincLossRadius = real(12 / binX) 
    ! Set up the background annulus
    
    calRadius =  CAL_omGetApertureRadius() / real(binx) 


    apertureRadius = real(12 / binx)
    innerRadius = calRadius + 2.0
    
    outerRadius = sqrt(areaRatio * apertureRadius * apertureRadius + innerRadius * innerRadius)


    call InitialiseArrays
 

    sortIndex => AllocateInt32(1, int(noSources, int32), "sortIndex (AperturePhotometry)")
     
    allocate(sourceCoords(1 : int(noSources, int32)), stat = status)
    call CheckAllocation(status, "sourceCoords (AperturePhotometry)")
 
    objectWidth => AllocateSingle(1, int(noSources, int32), "objectWidth (AperturePhotometry)")

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

end subroutine Initialise

!********************************************************
!
! Should be called at the very beginning of photometry
!
!*******************************************************
subroutine InitialiseArrays
  

    maxNumPixels = 30000
    if (associated(backgroundPixel)) return
  
    
    backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")


 
end subroutine InitialiseArrays
!********************************************************
!
! Should be called when finished doing photometry
!
!*******************************************************
subroutine deleteArrays

    if (associated(backgroundPixel)) then
        deallocate(backgroundPixel)
        nullify(backgroundPixel)
    end if


end subroutine DeleteArrays


!*************************************************************
!
! Point-source photometry
! Computes aperture photometry using the input radius
!
!*************************************************************
subroutine PointSourcePhotometry1(source, radius) 
    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius
    real(double) :: sourceCounts, backgroundCounts
    real(single) :: backgroundCountsError
    !real(double) :: bkgErrDouble

    real(double) :: backGroundCountsPerPixel
    real(single) r1, r2, area

    real(single) :: bkgAreaPix

    apertureRadius = radius

    !write(*, *) "L2436 PointSourcePhotometry1 radius=", radius, " x=", source%xpos, " y=", source%ypos

    area = PI * radius * radius ! The required aperture area

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    
    !write(*,*) "L2430 source innerBckRadius=", source%innerBckRadius, " outer=", source%outerBckRadius
    !write(*,*) "L2431 innerRadius=", innerRadius, " outerRadius=", outerRadius
    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if

    ! Get the total counts within the aperture      
    sourceCounts = GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, radius)
    !write(*,*) "L2443 within the aperture sourceCounts=", sourceCounts, "(total counts)"
    !write(*,*) "L2444 source rawBackgroundCountRate=", source%rawBackgroundCountRate
    !***********************************************************
    ! Get the background   
    !***********************************************************
    if (source%rawBackgroundCountRate .gt. -0.00001) then       
       backgroundCountsPerPixel = source%rawBackgroundCountRate
       !write(*,*) "L2451 backgroundCountsPerPixel=", backgroundCountsPerPixel
    else if(associated(backgroundImage)) then ! Use the background image
       !write(*,*) "L2452 bkgAreaPix=", bkgAreaPix
       backGroundCountsPerPixel = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
       r1, r2, backGroundCountsError, bkgAreaPix)
       !write(*,*) "L2454 bkgAreaPix=", bkgAreaPix
    else if (associated(sourceNumImage)) then
       !write(*,*) "L2456 associated sourceNumImage bkgAreaPix=", bkgAreaPix
       backGroundCountsPerPixel = GetBackGroundMode1(sourcenumImage, source%xPos - 0.5, source%yPos - 0.5, &
            r1, r2, backGroundCountsError, bkgAreaPix)
       !write(*,*) "L2469 backGroundCountsPerPixel =", backGroundCountsPerPixel
       !write(*,*) "L2460 bkgAreaPix=", bkgAreaPix
    else   
       !write(*,*) "L2461 bkgAreaPix=", bkgAreaPix
       backGroundCountsPerPixel = GetBackGroundMode3(source%xPos - 0.5, source%yPos - 0.5, &
            r1, r2, backGroundCountsError, bkgAreaPix)       
    end if
    backGroundCounts = backGroundCountsPerPixel * apertureArea
    !write(*,*) "L2521 backGroundCountsPerPixel=", backGroundCountsPerPixel, &
    !     " backGroundCounts=", backGroundCounts, " apertureArea=", apertureArea
    
    !write(*,*) "L2469 before scaling backGroundCountsError=", backGroundCountsError
    ! !!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 2013-10-15 the background count error is scaled to the measuring aperture area
    backGroundCountsError = backGroundCountsError * apertureArea 
    !write(*,*) "L2473 after scaling backGroundCountsError=", &
    !     backGroundCountsError, " apertureArea=", apertureArea
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    !*********************************************************************************
    ! 2013-10-27 vny: when the bkg. count error is scaled to the source aperture,
    ! this error becomes larger than the estimate sqrt(scaledBkgCounts). 
    ! This is not logical because the background was measured by smoothing 
    ! counts in a larger area, so the error in the source aperture area should be small
    !***********************************************************************************
    
    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    !write(*,*) "L2486 source rawBackGroundCountRate=", source%rawBackGroundCountRate
    source%rawBackGroundCountRateErr = backGroundCountsError / source%exposure
    !write(*,*) "L2487 source%rawBackGroundCountRateErr=", source%rawBackGroundCountRateErr
        
    ! Subtract the background and convert to count-rate
    sourceCounts = sourceCounts - backgroundcounts
    !write(*,*) "L2492 sourceCounts=", sourceCounts
    source%rawCountRate = sourceCounts / source%exposure
    
    !ttttttttttttttttttttttttttt 2013-11-26 computing by using the sqrt(s+2b)

    ! using sqrt(s+2b) formula
    source%rawCountRateErr = ComputeSourceRateError(sourceCounts, backgroundCounts, &
         source%exposure) 
    !write(*,*) "L2515 sqrt(s+2b) rawCountRateErr=", source%rawCountRateErr

    !ttttttttttttttttttttttttttttttt
    ! using Mat's formula
    source%rawCountRateErr = ComputeSourceRateError3(sourceCounts, source%rawBackGroundCountRate, &
         source%exposure, apertureArea, bkgAreaPix) 
    !write(*,*) "L2521 sqrt(s*t+b*t(1+sarea/barea)) rawCountRateErr=", source%rawCountRateErr
    !tttttttttttttttttttttttttttttttt

    ! 2013-10-15 vny: using the backgroundcounts error
    source%rawCountRateErr = ComputeSourceRateError(sourceCounts, backgroundCounts, &
         backGroundCountsError, source%exposure) 
    !write(*,*) "L2527 sqrt(s+b+b_err*b_err) rawCountRateErr=", source%rawCountRateErr


    !tttttttttttt
    source%corrCountRateErr=source%rawCountRateErr
    !write(*,*) "L2502 source corrCountRateErr=", source%corrCountRateErr

    !*****************************************************************************
    ! 2013-10-15 vny: using the background error estimate based on the bkg-sigma
    ! of the bkg-average value (before it was sigma-1 of a single bkg-measurement)
    !***************************************************************
    !**************
    !write(*,*) "L2509 calling ComputeSourceSignificance"
    source%Significance = ComputeSourceSignificance(source%rawCountRate, &
                      source%rawBackGroundCountRate, source%exposure)
    !source%Significance = ComputeSourceSignificance(source%rawCountRate, &
    !                  source%rawBackGroundCountRate, source%rawCountRateErr, &
    !                  source%rawBackGroundCountRateErr, source%exposure)
    !write(*,*) "L2515 source significance=", source%Significance
    !write(*,*) "L2516 "

end subroutine PointSourcePhotometry1

!*********************************************
! Point2SourcePhotometry
! Computes the counts for a point-source
! having an aperture radius > coincidence-loss
! radius
!*********************************************
subroutine PointSourcePhotometry2(source, radius)

    ! Input (unmodfied) variables
    real(single), intent(in) :: radius 
    type(SourceParameters), intent(inout) :: source
      
    real(double) :: rawCounts, correctedCounts, countRate
    real(double) :: f
    real(double) :: backgroundRate, backgroundCounts, backgroundMode
    real(single) :: backGroundCountsError
    real(double) :: area, pixelArea
    real(double) :: correction
 
    ! Local variables
    integer (int32) :: i, j
    integer (int32) :: i1, i2, j1, j2
    real(single) :: xPos, yPos, x0, y0 !   

    real(single) :: bkgAreaPix

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 10000000.0_double 
    maxExposure = 0.0_double
    


    xPos = source%xPos - 0.5
    yPos = source%yPos - 0.5
    apertureRadius = radius        
           
    ! Determine upper and lower array indices for
    ! summing counts
   
    i1 = int(xPos - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(xPos + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(yPos - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(yPos + radius + 0.0001)
    j2 = min(ju, j2)
 

    apertureArea = 0.0_double
    rawCounts = 0.0_double
    correctedCounts = 0.0_double
    area = 0.0_double
 
    do i = i1, i2
        x0 = real(i) + 0.5

        do j = j1, j2
            y0 = real(j) + 0.5 

           
            ! Compute the coincidence-loss correction for this pixel
            countRate = GetApertureCounts(x0, y0, coincLossRadius) / source%exposure
            apertureRadius = radius                   

 
            pixelArea = EffectivePixelArea(x0 - xPos, y0 - yPos)

            f = pixelArea * ccdImage(i, j) 


            !*********************************************************************
            ! Maybe using an exposure map- get the mean, min and max  exposure
            !********************************************************************  
            call getExposure(i, j)
  


            
            rawCounts = rawCounts + f
            correctedCounts = correctedCounts + f * &
                      CoincidenceLossCorrection(countRate, sqrt(real(countRate)), frameTime)
                   
            area = area + pixelArea
          
        end do
    end do


    ! Get the background   
    if (source%rawBackgroundCountRate .gt. -0.0001) then ! Use the sepecified value
        backgroundMode = source%rawBackgroundCountRate

    elseif(associated(backgroundImage)) then ! Use the background image
        
        backGroundMode = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
             !source%innerBckRadius, source%outerBckRadius, backGroundCountsError)
        source%innerBckRadius, source%outerBckRadius, backGroundCountsError, bkgAreaPix)
      
    else if (associated(sourceNumimage)) then  ! Use the image
        backGroundMode = GetBackGroundMode(sourceNumImage,               &
                               source%xPos - 0.5, source%yPos - 0.5,         &
                               source%innerBckRadius, source%outerBckRadius, &
                               !backGroundCountsError)
                               backGroundCountsError, bkgAreaPix)

    else
        backGroundMode = GetBackGroundMode(source%xPos - 0.5, source%yPos - 0.5, &
                               source%innerBckRadius, source%outerBckRadius,         &
                               !backGroundCountsError)
                               backGroundCountsError, bkgAreaPix)

    end if


    backgroundCounts = backGroundMode * area
    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    source%rawBackGroundCountRateErr = backGroundCountsError * area / source%exposure
  
    ! Subtract the background
    rawCounts = rawCounts - backGroundCounts 

  
    source%rawCountRateErr = ComputeSourceRateError(rawCounts, backGroundCounts, source%exposure)

  


    ! Compute the corrected rates
    backGroundRate = PI * coincLossRadius * coincLossRadius * backGroundMode / source%exposure

    ! The coincidence-loss correction for each background pixel
    correction = CoincidenceLossCorrection(backgroundRate, sqrt(real(backgroundRate)), frameTime)

    source%corrBackGroundCountRate = source%rawBackGroundCountRate * correction

    source%corrBackGroundCountRateErr = source%rawBackGroundCountRateErr * correction

    source%corrCountRate =  source%rawCountRate * correction
    
    source%corrCountRateErr = ComputeSourceRateError(correctedCounts, backgroundCounts, source%exposure)
    
    
    ! Compute the signal-to-noise (source counts / sqrt(source counts + background counts)   
   
    source%Significance = ComputeSourceSignificance(source%rawCountRate, &
                      source%rawBackGroundCountRate, source%exposure)
                  
    source%apertureRadius = radius * float(binX)  
    
    source%coincidenceLossCorr = (source%corrCountRate - source%rawCountRate) * 100.0 &
             / source%rawCountRate 
    source%rawCountRate = source%rawCountRate / source%exposure
    source%corrCountRate = source%corrCountRate / source%exposure

end subroutine PointSourcePhotometry2       
!*************************************************************
! Point-source photometry
!*************************************************************
subroutine PointSourcePhotometry3(source, radius, backgroundMode) 
    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius, backGroundMode
    real(double) :: sourceCounts, backgroundCounts
    real(single) :: backgroundCountsError
    
    real(double) :: backGroundCountsPerPixel
    real(single) r1, r2




    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
    


    apertureRadius = radius

    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if
  
    ! Get the total counts within the aperture   


    sourceCounts = GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, radius)
 

    ! Get the background   

    backGroundCountsPerPixel = backgroundMode

    backGroundCounts = backGroundCountsPerPixel * apertureArea

    source%rawBackGroundCountRate = backGroundCounts / source%exposure
    backGroundCountsError = backGroundCountsError * apertureArea 
  
    ! Subtract the background and convert to count-rate
    sourceCounts = sourceCounts - backgroundcounts
    source%rawCountRate = sourceCounts / source%exposure
    source%rawCountRateErr = ComputeSourceRateError(sourceCounts, backgroundCounts, source%exposure) 


end subroutine PointSourcePhotometry3


!*****************************************
! setQualityFlag
! Set quality-flag bit 0 (bad pixel)
!
!***************************************
subroutine setQualityFlag1

    currentSource%qualityFlag = ibset(currentSource%qualityFlag, 0)
    
end subroutine setQualityFlag1


!*****************************************
! setQualityFlag2
! Set quality-flag bit 0 (bad pixel)
!
!***************************************
subroutine setQualityFlag2(source)
    type(SourceParameters), intent(inout) :: source

    source%qualityFlag = ibset(source%qualityFlag, 0)
    
end subroutine setQualityFlag2

!**********************************
!
! subroutine GetCounts
!
!********************************** 
subroutine GetCounts1(image, source)
    real(single), pointer :: image(:, :)
    type(SourceParameters), intent(inout) :: source
!    real(single) :: trueCountRate
  
    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = source%apertureRadius
     
    source%rawBackgroundCountRate = &
             GetApertureCounts(source%xPos - 0.5, source%yPos - 0.5, apertureRadius )
               

    source%rawBackGroundCountRateErr = sqrt( source%rawBackgroundCountRate) 
 
    
    
   
    
end subroutine GetCounts1
!***************************************************
! subroutine GetCounts2
! Computes the total counts within a circle 
! of radius radius centred on xpos, ypos
!*************************************************** 
subroutine GetCounts2(image, xPos, yPos, radius, counts, area)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, radius
    real(single), intent(out) :: counts
    real(single), optional, intent(out) :: area

    CCDImage => image


    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = radius




    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  

     
    counts = GetApertureCounts(xPos, yPos, radius )
    nullify(CCDImage)           

    if (present(area)) area = apertureArea   ! The actual aperture area used

end subroutine GetCounts2

!**********************************
! subroutine GetBackgroundCounts1
!********************************** 
subroutine GetBackgroundCounts1(image, xPos, yPos, &
                                radius, radius1, radius2, counts)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, radius, radius1
    real(single), intent(inout) :: radius2
    real(single), intent(out) :: counts
    real(single) :: error
  
    real(single) :: bkgAreaPix

    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

    apertureRadius = radius

    maxNumPixels = 30000
    if (.not. associated(backgroundPixel)) then
        backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")
     
    end if


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  


    ! Compute the background counts per pixel from the annulus    
    counts = GetBackGroundMode3(   &
              xPos,    &   ! The centre x-pixel coordinate of the aperture
              yPos,    &   ! The centre y-pixel corrdinate of the aperture 
              radius1, &   ! The aperture inner radius
              radius2, &   ! The aperture outer radius
              !error)       ! The error
              error,   &       ! The error
              bkgAreaPix)
               



    counts = counts * PI * radius * radius


    deallocate(backgroundPixel)
    nullify(backgroundPixel)
    nullify(CCDImage)    

end subroutine GetBackgroundCounts1



!**********************************
! subroutine GetBackgroundCounts1
!********************************** 
subroutine GetBackgroundCounts2(image, xPos, yPos, &
                                area, radius1, radius2, counts)
    real(single), pointer :: image(:, :)
    real(single), intent(in) :: xPos, yPos, area, radius1
    real(single), intent(inout) :: radius2
    real(single), intent(out) :: counts
    real(single) :: error
  
    real(single) :: bkgAreaPix

    CCDImage => image

    nx = size(image, 1)
    ny = size(image, 2)

    xLeft = 0.0
    yBottom = 0.0
    xRight = real(nx)
    yTop = real(ny)
      
    il = 0
    iu = nx - 1
    jl = 0
    ju = ny - 1     

 

    maxNumPixels = 30000
    if (.not. associated(backgroundPixel)) then
        backgroundPixel => AllocateSingle(1, maxNumPixels, "backgroundPixel (AperturePhotometry)")
     
    end if


    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
  


    ! Compute the background counts per pixel from the annulus    
    counts = GetBackGroundMode3(   &
              xPos,    &   ! The centre x-pixel coordinate of the aperture
              yPos,    &   ! The centre y-pixel corrdinate of the aperture 
              radius1, &   ! The aperture inner radius
              radius2, &   ! The aperture outer radius
              !error)       ! The error
              error, &       ! The error
              bkgAreaPix)
               



    counts = counts * area


    deallocate(backgroundPixel)
    nullify(backgroundPixel)
    nullify(CCDImage)    

end subroutine GetBackgroundCounts2





!******************************************************
!
!
!
!
!*****************************************************

subroutine AperturePhotometry3(source, n, image, levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)

  type(SourceParameters), pointer :: source(:)
  integer, intent(in) :: n       
  real(single), pointer :: image(:, :) 
  integer(int16), pointer :: levelImage(:, :) 
  logical(boolean), pointer :: sourceQualityImage(:, :)
  
  real(single), intent(in) :: radius, minSignificance
  real(single) :: sigma = 0.0
  integer :: i
  
  if( associated(exposureImage) ) then
     source%exposure = 1.0
  end if
  !write(*,*) "L3006 aperturePhotometry3"
  qualityImage => sourceQualityImage ! The quality image (false=good pixels, true=bad pixels)
  
  call Initialise(int(size(source), int16), int(size(image, 1), int16), int(size(image, 2), int16))
  CCDImage => image 
  
  sourceNumImage => levelImage
    
  call refineSourceNumImage(source, int(n, int16), sourceNumimage)
  
  ! Compute the mode of the image, ignoring source regions
  backgroundCountsMode = ComputeBackgroundMode(image, &
       sourceNumImage, nx, ny, backgroundSigma, sigma)
  
  !write(*,*) "---"
  do i = 1, n
     !write(*,*) "L3022 i=", i    
     if (source(i)%extensionFlag .gt. 0) cycle

       if (.not. isNullValue(source(i)%significance) ) then
          if (source(i)%significance .gt. minSignificance) cycle
          if( source(i)%apertureRadius / real(binx) <= radius) cycle
       end if
       
       currentSource => source(i) 
       
       currentSource%innerBckRadius = 0.0
       currentSource%outerBckRadius = 0.0
       
    
       currentSource%rawBackgroundCountRate = -1.0


       if (associated(exposureImage) ) then    
          !write(*,*) "L3040 calling PointSourcePhotometry4, radius=", radius
          call PointSourcePhotometry4(currentsource, radius)
       else 
          !write(*,*) "L3042 calling PointSourcePhotometry1 radius=", radius
          call PointSourcePhotometry1(currentsource, radius)           
       end if
       
       currentsource%ApertureRadius = radius * real(binX)
       
       !***********************************************************************
       !
       ! If using an exposure image then multiply significance by sqrt(mean exposure)
       !
       !*************************************************************************
       !        if (associated(exposureImage) ) then
       !            call scaleErrors( currentSource )        
       !        end if
       !write(*,*) "L3057 calling  PointSourceCoincLossCorr"
       ! 2013-10-14 using deadFrac PointSourceCoincLossCorr"
       !call PointSourceCoincLossCorr( currentsource, 12.0, frameTime) 
       call PointSourceCoincLossCorr( currentsource, 12.0, frameTime, deadFrac)   
       
       nullify(currentSource)
 
    end do
    
    nullify(CCDImage, qualityImage, sourceNumImage, currentSource)
    call deleteArrays
    
    
end subroutine AperturePhotometry3




!**************************************************************
!
! Fow when using the background image for aperture photometry
!
!
!**************************************************************

subroutine AperturePhotometry4(source, n, image, &
                               backgroundImageIn, &
                               levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)

    type(SourceParameters), pointer :: source(:)
    integer, intent(in) :: n       
    real(single), pointer, dimension(:, :) :: image, backGroundImageIn 
    integer(int16), pointer :: levelImage(:, :) 
    logical(boolean), pointer :: sourceQualityImage(:, :)
 
    real(single), intent(in) :: radius, minSignificance

    backGroundImage => backgroundImageIn 

    !write(*,*) "L3103 AperturePhotometry4 radius=", radius, " calling AperturePhotometry3" 

    call AperturePhotometry3(source, n, image, levelImage, &
                               sourceQualityImage, &
                               radius, minSignificance)


    nullify(backgroundImageIn)

end subroutine AperturePhotometry4
!**********************************************************************
!
!
!
!
!*********************************************************************
subroutine refineSourceNumImage(source, nsources, levelimage)
    type(SourceParameters), pointer :: source(:)
    integer(int16), intent(in) :: nSources       
    integer(int16), pointer :: levelImage(:, :) 
    real(single) :: apertureRadius, radius, dx, dy

    integer(int16) :: i, j, k, i0, j0, il, iu, jl, ju, nx, ny, dx1


    
    nx = ubound(levelImage, 1)
    ny = ubound(levelImage, 2)


    apertureRadius = real(12 / binx)
    dx1 = 12 / binx + 1 

!
    where(levelimage > 0) levelImage = 0

    do i = 1, nSources
       
        if (source(i)%number < 0 .or. source(i)%extensionFlag == 1) cycle
        i0 = int(source(i)%xpos - 0.5)
        j0 = int(source(i)%ypos - 0.5)
        il = max(0_int16, i0 - dx1)
        iu = min(nx, i0 + dx1)
        jl = max(0_int16, j0 - dx1)
        ju = min(ny, j0 + dx1)


        do j = il, iu
            dx = source(i)%xpos - real(j + 1)
            if ( abs(dx) > apertureRadius) cycle
            do k = jl, ju
                if (levelImage(j, k) .ne. 0) cycle
                dy = source(i)%ypos - real(k + 1)
                if ( abs(dy) > apertureRadius) cycle
                radius = sqrt(dx * dx + dy * dy)
                if ( radius <= apertureRadius) then
                    levelImage(j, k) = source(i)%number
                end if 
         
             end do
        end do
 
    end do







!nx = nx + 1
!ny = ny + 1
!call Saveint16Image(levelImage, nx, ny, "levelimage.fits", "test")
!call fatal("stopped at line 3162")
end subroutine refineSourceNumImage


!**********************************************************
!
! Routine when exposure-image is being used to
! get the mean exposure within the aperture area
!
!
!**********************************************************

subroutine getExposure(i, j)
    integer, intent(in) :: i, j


    if ( .not. associated(exposureImage) ) return

    if( isNullValue( exposureImage(i, j) )) return

    numExposurePixels = numExposurePixels + 1
    meanExposure = meanExposure + dble(exposureImage(i, j ))
    minExposure = min( minExposure, dble(exposureImage(i, j ) ) )
    maxExposure = max( maxExposure, dble(exposureImage(i, j ) ) )

end subroutine getExposure
!***********************************************************************
! This routine has been added in July 2008 to scale the count-rate errors
! when an exposure image is being used. 
! In such cases the exposure time of
! each source will have been set to 1.
! The variable exposureConstant will have been set to the mean exposure of
! the exposure image.
! The variable meanExposure will equal the mean exposure within the
! aperture area
! NB source%exposure will have been set equal to 1.0
!
!
!***********************************************************************
subroutine scaleErrors( source )
    type(SourceParameters), intent(inout) :: source

    real(double) :: constant, constant1
 !   real(single) :: oldSignificance



    if (numExposurePixels .gt. 0) then

        

        !************************************************************************
        !
        ! Compute the mean exposure time within the photometry aperture
        !
        !***********************************************************************
        meanExposure = meanExposure / dble(numExposurePixels)



      
!oldSignificance = source%significance



        !**********************************************************************
        !
        ! Need to divide all rates by the exposure constant
        !
        ! Rate * (mean exposure / exposure constant) gives the total counts
        ! (since the source%exposure is set to 1.0) and then divide by the
        ! mean exposure to get the rate 
        ! equals rate / exposure constant 
        !
        !**********************************************************************
 
 

        source%rawCountRate = source%rawCountRate / exposureConstant 
        source%rawBackgroundCountRate = source%rawBackgroundCountRate / exposureConstant 

        !******************************************************************
        !
        ! Scale the significance
        !
        !******************************************************************
        constant = meanExposure * meanExposure / exposureConstant   
        constant1 = sqrt( constant )  



        source%significance = source%significance * constant / constant1  

        !******************************************************************
        !
        ! Scale the count-rate errors
        !
        !******************************************************************
        source%rawCountRateErr = source%rawCountRateErr / exposureConstant
        source%rawBackgroundCountRateErr = source%rawBackgroundCountRateErr / exposureConstant








        !***************************************************************************
        !
        ! Set the exposure-time equal to the mean exposure-time within the aperture
        !
        !***************************************************************************
        source%exposure = meanExposure
         




    end if


end subroutine scaleErrors


!*************************************************************
!
! Point-source aperture photometry (USING AN EXPOSURE IMAGE)
! 
! Input parameter
!        radius - aperture radius
! Input/output parameters
!        source - structure containing source-parameters (coordinates, etc)
!
! Notes
! Uses the EXPOSURE image
!
! The input image will originally have been in counts/second
! Omdetect will have multiplied it by the median exposure
!
! Method
! 1) Find the background counts per pixel, using the pixels within an annulus
! 
! 2) Sum the rate within the aperture - ie 
! 3) For each pixel-
!                 Subtract the background counts per pixel from the background
!                 Multiply this value by the exposure pixel value / divided by the mean exposure
!
!   
!*************************************************************
subroutine PointSourcePhotometry4(source, radius) 

    type(SourceParameters), intent(inout) :: source
    real(single), intent(in) ::radius
  
    
    real(single) :: backGroundCountsPerPixel,  backGroundCountsError 
    real(single) r1, r2, area

    real(single) :: bkgAreaPix
    real(double) :: sourceRateErr, bkgRateErr

    !write(*,*) "L3370 pointSourcePhotometry4"
    apertureRadius = radius

    area = PI * radius * radius ! The required aperture area
    !write(*,*) "L3376 radius=", radius, " area=", area
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    
    if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
        r1 = source%innerBckRadius
        r2 = source%outerBckRadius
    else 
        r1 =  innerRadius
        r2 =  outerRadius
    end if

    !write(*,*) "L3391 r1=", r1, " r2=", r2
    !***********************************************************
    !
    ! Get the background
    !   
    !***********************************************************
    if (source%rawBackgroundCountRate .gt. -0.00001) then
        backgroundCountsPerPixel = source%rawBackgroundCountRate
        !write(*,*) "L3399 source rawBackgroundCountRate=", source%rawBackgroundCountRate
       
    else if(associated(backgroundImage)) then ! Use the background image
        
        backGroundCountsPerPixel = GetBackGroundMode4(backgroundImage, source%xPos - 0.5, source%yPos - 0.5, &
             !r1, r2, backGroundCountsError)
             r1, r2, backGroundCountsError, bkgAreaPix)
        !write(*,*) "L3406 backGroundCountsPerPixel=", backGroundCountsPerPixel
        
    else if (associated(sourceNumImage)) then

        backGroundCountsPerPixel = GetBackGroundMode1(sourcenumImage, source%xPos - 0.5, source%yPos - 0.5, &
             r1, r2, backGroundCountsError, bkgAreaPix)
        !write(*,*) "L3413 backGroundCountsPerPixel=", backGroundCountsPerPixel, " backGroundCountsError=", &
        !     backGroundCountsError
    else   
        backGroundCountsPerPixel = GetBackGroundMode3(source%xPos - 0.5, source%yPos - 0.5, &                            
             r1, r2, backGroundCountsError, bkgAreaPix)
        !write(*,*) "L3419 backGroundCountsPerPixel=", backGroundCountsPerPixel, " backGroundCountsError=", &
        !     backGroundCountsError
    end if
    
    !write(*,*) "L3423 calling GetApertureCountrate"
    !***********************************************
    !
    ! backgroundCountsPerPixel is for an exposure of exposureConstant
    ! Get the background-subtracted raw count-rate and  
    ! the also background raw-rate
    !**********************************************
    call GetApertureCountrate(                         &
                    radius,                            &
                    backgroundCountsPerPixel,          &  
                    backGroundCountsError,             &
                    source)
    
    source%exposure = meanExposure 
    !write(*,*) ".."
    !write(*,*) "L3433 source Exposure=", source%exposure 
    !write(*,*) "L3436 source%rawCountRate=", source%rawCountRate, " source%rawCountRateErr=", &
    !     source%rawCountRateErr
    !write(*,*) "L3438 source%rawbackgroundCountRate=", source%rawbackgroundCountRate, &
    !     " source%rawBackgroundCountRateErr=", source%rawBackgroundCountRateErr
    sourceRateErr = computeSourceRateError(dble(source%rawCountRate*source%exposure), &
         dble(source%rawbackgroundCountRate*source%exposure), source%exposure ) 
    !write(*,*) "L3443 sourceRateErr=", sourceRateErr
    bkgRateErr=dble(sqrt(source%rawbackgroundCountRate*source%exposure)/source%exposure)
    !write(*,*) "L3445 bkgRateErr=", bkgRateErr
    source%rawCountRateErr=sourceRateErr
    source%rawBackgroundCountRateErr=bkgRateErr
end subroutine PointSourcePhotometry4


!*********************************************
!
! subroutine GetApertureCountRate
! For use with an exposure image and for point sources
! Computes the counts within a given aperture
! 
!
!*********************************************
subroutine      GetApertureCountRate(  &
                radius,                        & ! The aperture radius (pixels) for summing the counts
                backgroundCountsPerPixel,      & ! The background counts per pixel from the annulus
                backgroundCountsPerPixelError, & ! The background counts per pixel error from the annulus
                source)             


    !********************************************* 
    !
    ! Input (unmodfied) variables
    !*********************************************
    real(single), intent(in) :: radius
    real(single), intent(in) :: backgroundCountsPerPixel, backgroundCountsPerPixelError 
   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source

    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant, area   
    real(double) :: sum1, sum2, sum3, sum4, totalArea
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0
    real(double) :: nSummedPixels



!ccdImage = 10.0
!exposureImage = 1000.0
!exposureConstant = 1000.0 
    apertureRadius = radius  ! For internal use      
           
    ! Determine upper and lower array indices for
    ! summing counts
    
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5

    !write(*,*) "L3499 x0=", x0, " y0=", y0, " apertureRadius=", apertureRadius

    i1 = int(x0 - radius - 0001)
    i1 = max(il, i1)
  
    i2 = int(x0 + radius + 0.0001)
    i2 = min(iu, i2)  

    j1 = int(y0 - radius - 0.0001)
    j1 = max(jl, j1)

    j2 = int(y0 + radius + 0.0001)
    j2 = min(ju, j2)

    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double
    !write(*,*) "L3517 i1=", i1, " i2=", i2, " j1=", j1, " j2=", j2
    !**************************************************
    !
    ! Sum the pixels in the aperture round the centre
    !
    !**************************************************
    !write(*,*) "L3523 exposureImage(i2,j2)=", exposureImage(i2,j2)
    numExposurePixels = 0
    nSummedPixels=0.0_double
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
    !write(*,*) "L3525 exposureConstant=", exposureConstant
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture

    totalArea = 0.0_double
    do i = i1, i2
        do j = j1, j2         

           !write(*,*) "L3539 ..."
            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue(ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value (multiplied by the area of the pixel- 
            ! 1 if totally inside aperture 0 if totally outside aperture)
            ! Remember that the original image had been divided by the total exposure
            ! and omdetect has multiplied it by exposureConstant
            ! area is returned and is the actual pixel area (0-1)
            !*************************************************************
            !write(*,*) "L3566 x0=", x0, " y0=", y0, " i=", i, " j=", j
            imageCounts = PixelValue1(x0, y0, i, j, area) * constant ! The pixel counts -
            !write(*,*) "L3568 x0=", x0, " y0=", y0, " i=", i, " j=", j, "imageCounts=", &
            !     imageCounts, " constant=", &
            !     constant, " exposureConstant=", exposureConstant
 
            if(area .lt. 1e-6) cycle
            !write(*,*) "L3572  i=", i, " j=", j

            totalArea = totalArea + area 
            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = backgroundCountsPerPixel * area * constant
                
            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)
!            backgroundSubtractedCounts = imageCounts - backgroundCounts

            sum4 = sum4 + backgroundSubtractedCounts
            !write(*,*) "L3595 imageCounts=", imageCounts, " backgroundCounts=", backgroundCounts, &
            !     " sum3=", sum3, " sum4=", sum4
            
            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )

            !write(*,*) "L3598 sourceRate=", sourceRate
            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                    dble(exposureImage(i, j) )
 

            !write(*,*) "L3608 backgroundRate=", backgroundRate

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                                   backgroundCounts, exposureImage(i, j) ) 
            !write(*,*) "L3622 sourceRateError=",  sourceRateError, " exposureImage=", exposureImage(i, j)
   
            sum1 = sum1 + sourceRateError * sourceRateError 
            !write(*,*) "L3620 sum1=", sum1, " constant=", constant, " area=", area
            !write(*,*) "L3621 backgroundCountsPerPixelError=",  backgroundCountsPerPixelError
            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCountsPerPixelError * area * constant / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError
            !write(*,*) "L3629 sum2=", sum2
            
            numExposurePixels = numExposurePixels + 1
            
            meanExposure = meanExposure + dble(exposureImage(i, j))
            nSummedPixels=nSummedPixels+1.0_double
            !write(*,*) "L3637 i=", i, " j=", j, " nSummedPixels=", nSummedPixels, &
            !     " exposureImage(i,j)=", exposureImage(i,j)

            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 
            

            !*******************************************************
            ! Check the qualaity pixel
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                call setQualityFlag(source)    
            end if
        end if

     
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate

    source%rawbackgroundCountRate = backgroundRate
    source%corrbackgroundCountRate = backgroundRate

    !write(*,*) "L3663 minExposure=", minExposure, " maxExposure=", maxExposure
    !write(*,*) "L3664 meanExposure=", meanExposure

    ! 2013-12-12 correcting the calculation of the mean exposure
    ! (it was just the sum without division by number of summed pixels)
    if (nSummedPixels .gt. 1.0_double) then
       meanExposure=meanExposure/nSummedPixels
    endif
    !write(*,*) "L3673 meanExposure=", meanExposure
    !write(*,*) "L3674 sourceRate=", sourceRate, " backgroundRate=", backgroundRate

    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr
    
    !write(*,*) "L3677 source rawCountRAteErr=", source%rawCountRateErr

    source%rawBackgroundCountRateErr = sqrt(sum2)
    source%CorrBackgroundCountRateErr = source%rawBackgroundCountRateErr
    !write(*,*) "L3681 source rawBackgroundCountRateErr=", source%rawBackgroundCountRateErr

    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    !write(*,*) "L3685 sum3=", sum3, " sum4=", sum4
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if

    !write(*,*) "L3698 sum4=", sum4, " sourceRate=", sourceRate
    !write(*,*) "L3699 source exposure=",source%exposure 

    if( (maxExposure - minExposure) > 10.0) then
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if

    apertureArea = totalArea

end subroutine GetApertureCountRate     




!*************************************************************
!
! Extended-source photometry (USING AN EXPOSURE IMAGE)
! 
! Input parameter
!        radius - aperture radius
! Input/output parameters
!        source - structure containing source-parameters (coordinates, etc)
!
! Notes
! Uses the EXPOSURE image
!
! The input image will originally have been in counts/second
! Omdetect will have multiplied it by the median exposure
!
! Method
! 1) Find the background counts per pixel, using the pixels within an annulus
! 
! 2) Sum the rate within the aperture - ie 
! 3) For each pixel-
!                 Subtract the background counts per pixel from the background
!                 Multiply this value by the exposure pixel value / divided by the mean exposure
!
!   
!*************************************************************
subroutine ExtendedSourcePhotometry4(source) 

    type(SourceParameters), intent(inout) :: source
   
  
    real(single) :: backGroundCountsPerPixel,  backGroundCountsError 
    real(single) r1, r2, area, theta

    real(single) :: bkgAreaPix

    area = PI * source%semiA * source%semiB ! The required aperture area
    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 100000000.0_double 
    maxExposure = 0.0_double
    




    !***********************************************
    !
    ! backgroundCountsPerPixel is for an exposure of exposureConstant
    ! Get the background-subtracted raw count-rate and  
    ! the also background raw-rate
    !**********************************************

    if( associated(backgroundImage) ) then
         call GetExtendedCountrate2( source )

    else

        if (source%innerBckRadius > 0.0 .and. source%outerBckRadius > 0.0) then
            r1 = source%innerBckRadius
            r2 = source%outerBckRadius
        else 
            r1 =  source%semib  + real(4 / binx)
            r2 =  source%semib  + real(10 / binx)
        end if


        theta = source%theta
        if(theta < 0.0) theta = theta + 180.0

 
        source%axialRatio = source%semiA / source%semiB
        backgroundCountsPerPixel = GetBackGroundMode2( &
                     source%xPos,                    &   ! The centre x-pixel coordinate of the aperture
                     source%yPos,                    &   ! The centre y-pixel corrdinate of the aperture 
                     r1,                             &   ! The aperture inner radius
                     r2,                             &   ! The aperture outer radius
                     source%axialRatio,              &   ! The ellipse axial ratio
                     theta,                          &   ! The ellipse position angle
                     !backgroundCountsError)      ! The error
                     backgroundCountsError,          &      ! The error
                     bkgAreaPix)                   ! background area in pixels
        
         call GetExtendedCountrate1(                   &
                    backgroundCountsPerPixel,          &  
                    backGroundCountsError,             &
                    source)
 
    end if

    source%exposure = meanExposure 


end subroutine ExtendedSourcePhotometry4

!*********************************************
!
! subroutine GetExtendedCountRate
! For use with an exposure image
! Computes the counts within a given aperture
! 
!
!*********************************************
subroutine      GetExtendedCountRate1(         &
                backgroundCountsPerPixel,      & ! The background counts per pixel from the annulus
                backgroundCountsPerPixelError, & ! The background counts per pixel error from the annulus
                source)             


    !********************************************* 
    !
    ! Input (unmodfied) variables
    !********************************************
    real(single), intent(in) :: backgroundCountsPerPixel, backgroundCountsPerPixelError 
   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source


    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant
    real(double) :: sum1, sum2, sum3, sum4
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0




           
    ! Determine upper and lower array indices for
    ! summing counts
   
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5


    i1 = source%imin
    i1 = max(il, i1)
  
    i2 = source%imax
    i2 = min(iu, i2)  

    j1 = source%jmin
    j1 = max(jl, j1)

    j2 = source%jMax
    j2 = min(ju, j2)




    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double

    !**************************************************
    !
    ! Sum the pixels in the strip round the centre
    !
    !**************************************************

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture


    do i = i1, i2
        do j = j1, j2
                   
            if (sourceNumImage(i, j) .ne. source%number) then ! The pixel is not part of the sources
                cycle
            end if
            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue( ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value 
            !
            !*************************************************************
            imageCounts = dble(ccdImage(i, j)) * constant ! The pixel counts -
             


            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = backgroundCountsPerPixel * constant
                

            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)

!            backgroundSubtractedCounts = imageCounts - backgroundCounts


            sum4 = sum4 +  backgroundSubtractedCounts
    
            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )


            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                dble(exposureImage(i, j) )

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                               backgroundCounts, exposureImage(i, j) ) 
                
            sum1 = sum1 + sourceRateError * sourceRateError 

            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCountsPerPixelError * constant / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError

            numExposurePixels = numExposurePixels + 1

     

            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 


            !*******************************************************
            !
            ! Check the quality pixel
            !
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                    call setQualityFlag(source)    
                end if
            end if





        
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate


    source%RawbackgroundCountRate = backgroundRate
    source%CorrbackgroundCountRate = backgroundRate

    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr 
 

    source%rawBackgroundCountRateErr = sqrt(sum2)

    source%corrBackgroundCountRateErr =  source%rawBackgroundCountRateErr


    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

 

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if
    if( (maxExposure - minExposure) > 10.0) then    
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if

end subroutine GetExtendedCountRate1     





!*********************************************
!
! subroutine GetExtendedCountRate
! For use with an exposure image
! Computes the counts within a given aperture
! 
! For use when a background image is available
!*********************************************
subroutine  GetExtendedCountRate2( source )             

   
    !********************************************* 
    !
    ! Input/output (unmodfied) variables
    !*********************************************
    type(SourceParameters), intent(inout) :: source


    !****************************************
    ! Local variables
    !****************************************
    integer(int32) :: i, j
    integer(int32) :: i1, i2, j1, j2
    real(double) :: imageCounts, backgroundCounts, backgroundSubtractedCounts
    real(double) :: constant
    real(double) :: sum1, sum2, sum3, sum4
    real(double) :: sourceRate, backgroundRate 
    real(double) :: sourceRateError, rateError
    real(single) :: x0, y0




           
    ! Determine upper and lower array indices for
    ! summing counts
   
    x0 = source%xpos - 0.5
    y0 = source%ypos - 0.5


    i1 = source%imin
    i1 = max(il, i1)
  
    i2 = source%imax
    i2 = min(iu, i2)  

    j1 = source%jmin
    j1 = max(jl, j1)

    j2 = source%jMax
    j2 = min(ju, j2)




    backgroundRate = 0.0_double
    sourceRate = 0.0_double ! Initialise to zero 

    apertureArea = 0.0_double

    !**************************************************
    !
    ! Sum the pixels in the strip round the centre
    !
    !**************************************************

    numExposurePixels = 0
    meanExposure = 0.0_double
    minExposure = 1000000.0_double 
    maxExposure = 0.0_double
   
    sum1 = 0.0_double ! For storing the sum of the error squared of the background-subtracted raw count-rate
    sum2 = 0.0_double ! For storing the sum of the error squared of the background raw count-rate

    sum3 = 0.0_double ! For storing the total image counts within the aperture
    sum4 = 0.0_double ! For storing the total background-subtracted counts within the aperture


    do i = i1, i2
        do j = j1, j2
                   
            if (sourceNumImage(i, j) .ne. source%number) then ! The pixel is not part of the sources
                cycle
            end if
            !**********************************************************
            !
            ! Check the exposure pixel
            !
            !**********************************************************
            if ( isNullValue( exposureImage(i, j) ) .or. &
                 isNullvalue( ccdImage(i, j) ) ) then
                cycle
            end if

            !************************************************************************
            !
            ! The following corrects for the true exposure
            !
            !************************************************************************
            constant = dble(exposureImage(i, j) ) /  exposureConstant 


            !*************************************************************
            !
            ! Get the pixel value 
            !
            !*************************************************************
            imageCounts = dble(ccdImage(i, j)) * constant ! The pixel counts -
             


            sum3 = sum3 + imageCounts 

            !***************************************************  
            !              
            ! Compute the background counts
            ! 
            !***************************************************
            backgroundCounts = dble(backgroundImage(i, j)) * constant
                

            !***************************************************  
            !              
            ! Compute the background subtrated counts 
            ! 
            !***************************************************
            backgroundSubtractedCounts = max(0.0_double, imageCounts - backgroundCounts)




            sum4 = sum4 + backgroundSubtractedCounts
    
            !************************************************************************
            !
            ! Convert background-subtracted  counts to count-rate and add to the sum 
            !
            !************************************************************************
            sourceRate = sourceRate + backgroundSubtractedCounts / &
                           dble(exposureImage(i, j) )


            !************************************************************************
            !
            ! Convert background counts to count-rate and add to the sum 
            !
            !************************************************************************
            backgroundRate = backgroundRate + backgroundcounts / &
                                dble(exposureImage(i, j) )

            !************************************************************************
            !
            ! Compute the background-subtracted count-rate error and add the square
            ! of it to sum1
            !************************************************************************
            sourceRateError = computeSourceRateError( backgroundSubtractedCounts, &
                               backgroundCounts, exposureImage(i, j) ) 
                
            sum1 = sum1 + sourceRateError * sourceRateError 

            !***************************************************************************
            !
            ! Compute the background count-rate error and add the square of it to sum2 
            !
            !***************************************************************************
            rateError = backgroundCounts / dble(exposureImage(i, j)) 
            sum2 = sum2 + rateError * rateError

            numExposurePixels = numExposurePixels + 1

     

            minExposure = min(minExposure, dble(exposureImage(i, j)) ) 
            maxExposure = max(maxExposure, dble(exposureImage(i, j)) ) 


            !*******************************************************
            !
            ! Check the quality pixel
            !
            !******************************************************
            if (associated(qualityImage) ) then    
                if (qualityImage(i, j)) then ! Bad pixel- set source quality flag
                    call setQualityFlag( source )    
                end if
            end if

        
        end do

    end do

    source%rawCountRate = sourceRate
    source%CorrCountRate = sourceRate


    source%RawbackgroundCountRate = backgroundRate
    source%CorrbackgroundCountRate = backgroundRate

    !*********************************************************
    !
    ! The errors in the raw rates
    !
    !**********************************************************
    source%rawCountRateErr = sqrt(sum1)
    source%CorrCountRateErr = source%rawCountRateErr 
 

    source%rawBackgroundCountRateErr = sqrt(sum2)

    source%corrBackgroundCountRateErr =  source%rawBackgroundCountRateErr


    !****************************************************************
    !
    ! Compute the significance (signal-to-noise)
    !
    !******************************************************************
    if(sum3 > 0.0) then
        source%significance = sum4 / sqrt(sum3)
    else
        source%significance = 0.0
    end if

 

    if(sourceRate > 0.0) then
        source%exposure = sum4 / sourceRate
    end if
    if( (maxExposure - minExposure) > 10.0) then    
        source%qualityFlag = ibset( source%qualityFlag, 10)
    end if

end subroutine GetExtendedCountRate2     

end module omPhotometry_mod

























