!
! Module      : splinefit_mod
! Authors     : Chris Brindle (MSSL)
!
! Description : This module contains a routine for fitting a cubic-spline
!               function to a data set, and one for interpolation.
!                
!
! Comments    : 
!
! History : Created  : May 2001 - CB
!           Modified : 

module splinefit_mod
!use dal
use errorHandling
use omallocation_mod
use omnumerics_mod
implicit none



public


!**********************************************************************
! One-dimensional spline
!*********************************************************************

type OneDSpline
    integer :: size
    real(double), pointer :: x(:) => null()
    real(double), pointer ::  y(:) => null() 
  
    real(double), pointer :: y2(:) => null()
end type OneDSpline

!**********************************************************************
! Two-dimensional spline
!*********************************************************************
type TwoDSpline
    integer :: nx, ny
    real(double), pointer :: x(:) => null()
    real(double), pointer :: y(:) => null()
    real(double), pointer :: f(:, :) => null()
    real(double), pointer :: y2a(:, :) => null()

end type TwoDSpline

interface Constructspline
module procedure OneDSplineConstruct1, OneDSplineConstruct2, TwoDSplineConstruct



end interface ConstructSpline
interface DestructSpline
module procedure OneDSplineDestruct, TwoDSplineDestruct



end interface DestructSpline
interface InterpolateSpline


module procedure OneDSplineInterpolate1, OneDSplineInterpolate2, TwoDSplineInterpolate
end interface InterpolateSpline
   


contains


!**********************************************************
!
!**********************************************************
subroutine OneDSplineConstruct1(n, x, y, splineData)
    integer, intent(in) :: n
    real(single), dimension(1 : n) :: x, y
    type(OneDSpline), intent(out) :: splineData
   

    splineData%size = n
    if (associated(splineData%x)) deallocate(splineData%x) 
     
    splineData%x => AllocateDouble(1, n, "x")
  
    if (associated(splineData%y)) deallocate(splineData%y) 
 
    splineData%y => AllocateDouble(1, n, "y")

 
 
    if (associated(splineData%y2)) deallocate(splineData%y2) 
 
    splineData%y2 => AllocateDouble(1, n, "y2")
 
    splineData%x = dble(x)
    splineData%y = dble(y)
   

    call SplineFit(splineData%x, splineData%y, n, splineData%y2)



end subroutine OneDSplineConstruct1

!**********************************************************
!
!**********************************************************
subroutine OneDSplineConstruct2(n, x, y, splineData)
    integer, intent(in) :: n
    real(double), dimension(1 : n) :: x, y
    type(OneDSpline), intent(out) :: splineData
   

    splineData%size = n
    if (associated(splineData%x)) deallocate(splineData%x) 
     
    splineData%x => AllocateDouble(1, n, "x")
  
    if (associated(splineData%y)) deallocate(splineData%y) 
 
    splineData%y => AllocateDouble(1, n, "y")

 
 
    if (associated(splineData%y2)) deallocate(splineData%y2) 
 
    splineData%y2 => AllocateDouble(1, n, "y2")
 
    splineData%x = x
    splineData%y = y


    call SplineFit(x, y, n, splineData%y2)



end subroutine OneDSplineConstruct2





!**********************************************************
!
!**********************************************************
subroutine OneDSplineDestruct(splineData)
    
    type(OneDSpline), intent(inout) :: splineData

    
    deallocate(splineData%x)
    nullify(splineData%x)
    deallocate(splineData%y)
    nullify(splineData%y)
 
    deallocate(splineData%y2)
    nullify(splineData%y2)


end subroutine OneDSplineDestruct

!**********************************************************
!
!**********************************************************


real(double) Function OneDSplineInterpolate1(x, splineData)
    real(single), intent(in) :: x    
    type(OneDSpline), intent(in) :: splineData
     OneDSplineInterpolate1 = &
           OneDSpline_Interpolate(splineData%x, splineData%y, splineData%y2, splineData%size, dble(x))
end Function  OneDSplineInterpolate1


!**********************************************************
!
!**********************************************************


real(double) Function OneDSplineInterpolate2(x, splineData)
    real(double), intent(in) :: x    
    type(OneDSpline), intent(in) :: splineData
     OneDSplineInterpolate2 = &
           OneDSpline_Interpolate(splineData%x, splineData%y, splineData%y2, splineData%size, x)
end Function  OneDSplineInterpolate2







!****************************************************************
! Subroutine Spline_Fit
!****************************************************************
subroutine SplineFit(x, y, n, y2)
    integer, intent(in) :: n
    real(double), dimension(:) :: x, y
    real(double), dimension(:) :: y2
    integer :: i, k, status
    real(double) :: p, sig  
    real(double), dimension(:), allocatable :: u


    allocate(u(1 : n - 1), stat = status)
 
    y2(1) = 0.0
    u(1) = 0.0

    do i = 2, n - 1
        sig = (x(i) - x(i - 1)) / (x(i + 1) - x(i - 1))
        p = sig * y2(i - 1) + 2.0
        y2(i) = (sig - 1.0) / p
        u(i) = (y(i + 1) - y(i)) / (x(i + 1) - x(i)) - (y(i) - y(i - 1)) / (x(i) - x(i - 1))
        u(i) = (6.0 * u(i) / (x(i + 1) - x(i - 1)) - sig * u(i - 1)) / p;
    end do

    y2(n) = 0.0;
    do k = n - 1, 1, -1
        y2(k) = y2(k) * y2(k + 1) + u(k)
    end do
    
    deallocate(u)
end subroutine SplineFit


!*****************************************
! Function Spline_Interpolate
!*****************************************
real(double) function OneDSpline_Interpolate(xa, ya, y2a, n, x)

    integer, intent(in) :: n
    real(double), pointer, dimension(:) :: xa, ya, y2a
    real(double), intent(in) :: x
    integer :: klo, khi
    real(double) ::  h, b, a

    
    khi = 1
    klo = max(1, Locate(n, xa, x, khi))
    if(klo .eq. n) klo = max(1, klo - 1)
    khi = klo + 1


    h = xa(khi) - xa(klo)
    a = (xa(khi) - x) / h
    b = (x - xa(klo)) / h

    OneDSpline_Interpolate = (a * ya(klo) + b * ya(khi) + ((a * a * a - a) * y2a(klo) + (b * b * b - b) &
         * y2a(khi)) * (h * h) / 6.0)

end function OneDSpline_Interpolate    


!************************************************************************
! Two-dimension spline
!***********************************************************************




subroutine TwoDSplineConstruct(nx, ny, x, y, f, splineData)
    integer, intent(in) :: nx, ny
    real(single), pointer, dimension(:) :: x
    real(single), pointer, dimension(:) :: y
    real(single), pointer, dimension(:, : ) :: f


    type(TwoDSpline), intent(out) :: splineData
    

    splineData%nx = nx
    splineData%ny = ny
   
    if (associated(splineData%x)) deallocate(splineData%x) 
  
    splineData%x => AllocateDouble(1, nx, "x")
  
    if (associated(splineData%y)) deallocate(splineData%y) 
 
    splineData%y => AllocateDouble(1, ny, "y")
    if (associated(splineData%f)) deallocate(splineData%f) 
 
    splineData%f => Allocate2DDouble(0, nx - 1, 0, ny - 1,  "f")
 
    if (associated(splineData%y2a)) deallocate(splineData%y2a) 
 
    splineData%y2a => Allocate2DDouble(0, nx - 1, 0, ny - 1,  "y2a")
  

    splineData%x = dble(x)
    splineData%y = dble(y)
    splineData%f = dble(f)

  
    call TwoDSplineFit(nx, ny, splineData%x, splineData%f, splineData%y2a)
   
    
end subroutine TwoDSplineConstruct

!**********************************************************
!
!**********************************************************
subroutine TwoDSplineDestruct(splineData)
    
    type(TwoDSpline), intent(inout) :: splineData

    
    deallocate(splineData%x)
    nullify(splineData%x)
    deallocate(splineData%y)
    nullify(splineData%y)
    deallocate(splineData%f)
    nullify(splineData%f)
    deallocate(splineData%y2a)
    nullify(splineData%y2a)
    




end subroutine TwoDSplineDestruct

!**********************************************************
!
!**********************************************************
real(double) Function TwoDSplineInterpolate(x, y, splineData)
    real(double), intent(in) :: x, y    
    type(TwoDSpline), intent(in) :: splineData

     TwoDSplineInterpolate = &
         TwoDSpline_Interpolate1(x, y, splineData%nx, splineData%ny, &
           splineData%x, splineData%y, splineData%f, splineData%y2a)


end Function  TwoDSplineInterpolate

!**********************************************************
!
!**********************************************************
Subroutine TwoDSplineFit(nx, ny, x, ya, y2a)
    integer, intent(in) :: nx, ny
    real(double), pointer :: x(:)
    real(double), pointer :: ya(:, :)
    real(double), pointer :: y2a(:, :)

    real(double), pointer :: yTemp(:)=>null(), yTemp1(:)=>null() 
    integer :: j, k
    allocate(yTemp(1 : nx))
    allocate(yTemp1(1 : nx))

    do j = 1, ny
        forall (k = 1 : nx) yTemp(k) = ya(k - 1, j - 1)
             
        call SplineFit(x, yTemp, nx, yTemp1)
        forall (k = 1 : nx) y2a(k - 1, j - 1) = yTemp1(k)
    end do
    deallocate(yTemp, yTemp1)
end subroutine TwoDSplineFit


!**********************************************************
!
!**********************************************************
real(double) Function  TwoDSpline_Interpolate(xI, yI, nx, ny, x, y, ya, y2a)
    real(double), intent(in) :: xI, yI ! Point to be interpolated
    integer, intent(in) :: nx, ny
    real(double), pointer :: x(:), y(:)
    real(double), pointer, dimension(:, :) :: ya, y2a
    
    real(double), pointer:: yyTemp(:)=>null(), yTemp(:)=>null()
    real(double), pointer :: yaTemp(:)=>null(), y2aTemp(:)=>null() 
    real(double), pointer :: x1(:)=>null(), y1(:)=>null()
    integer :: j, k
    
    ! Evaluate the row splines at x
    allocate(yyTemp(1 : ny))
    allocate(yTemp(1 : ny))
    allocate(y1(1 : ny))
    allocate(x1(1 : nx))
 
    allocate(yaTemp(1 : nx))
    allocate(y2aTemp(1 : nx))


    do j = 1, ny
        
        forall (k = 1 : nx)  
            
            yaTemp(k) = ya(k - 1, j - 1)
            y2aTemp(k) = y2a(k - 1, j - 1)

        end forall 
       
        yyTemp(j) = OneDSpline_Interpolate(x, yaTemp, y2aTemp, nx, xI)
    end do

    ! Construct the one-d column spline
    call SplineFit(y, yyTemp, ny, yTemp)

    ! Evaluate the spline at y
    TwoDSpline_Interpolate = OneDSpline_Interpolate(y, yyTemp, yTemp, ny, yI)
    deallocate(x1, y1, yyTemp, yTemp, yaTemp, y2aTemp)
end function TwoDSpline_Interpolate


!**********************************************************
!
!**********************************************************
real(double) Function  TwoDSpline_Interpolate1(xI, yI, nx, ny, x, y, ya, y2a)
    real(double), intent(in) :: xI, yI ! Point to be interpolated
    integer, intent(in) :: nx, ny
    real(double), pointer :: x(:), y(:)
    real(double), pointer, dimension(:, :) :: ya, y2a
    
    real(double), pointer, dimension(:) :: yyTemp=>null(), yTemp=>null()
    real(double), pointer, dimension(:) :: yaTemp=>null(), y2aTemp=>null() 
    real(double), pointer :: x1(:)=>null(), y1(:)=>null()
    integer :: j0, k0, j, k, j1, j2, j3, k1, k2, nx1, ny1
   
    integer, parameter :: width = 10

    ! Evaluate the row splines at x
    allocate(yyTemp(1 : ny))
    allocate(yTemp(1 : ny))
    allocate(y1(1 : ny))
    allocate(x1(1 : nx))
 
    allocate(yaTemp(1 : nx))
    allocate(y2aTemp(1 : nx))



    j0 = 1
    j0 =  Locate(ny, y, yi, j0)
    j0 = max(0, j0)
    j0 = min(ny, j0)
    j1 = max(0, j0 - width)
    j2 = min(ny - 1, j0 + width)
    k0 = 1
    k0 =  Locate(nx, x, xi, k0)
    k0 = max(0, k0)
    k0 = min(ny - 1, k0)
  

    k1 = max(0, k0 - width)
    k2 = min(nx - 1, k0 + width)
    k0 = 1
    k0 = Locate(nx, x, xi, k0)
    k0 = max(0, k0)
    k0 = min(ny, k0)
    ny1 = j2 - j1 + 1
    nx1 = k2 - k1 + 1

    do j = 1, ny1
        y1(j) = y(j1 + j) 
        j3 = j1 + j - 1
        do k = 1, nx1  
            x1(k) = x(k1 + k) 
            yaTemp(k) = ya(k1 + k - 1, j3)
            y2aTemp(k) = y2a(k1 + k - 1, j3)

        end do
       
        yyTemp(j) = OneDSpline_Interpolate(x1, yaTemp, y2aTemp, nx1, xI)
    end do

    ! Construct the one-d column spline
    call SplineFit(y1, yyTemp, ny1, yTemp)

    ! Evaluate the spline at y
    TwoDSpline_Interpolate1 = OneDSpline_Interpolate(y1, yyTemp, yTemp, ny1, yI)
    deallocate(x1, y1, yyTemp, yTemp, yaTemp, y2aTemp)

end function TwoDSpline_Interpolate1

end module splinefit_mod















